import sys
from datetime import datetime
from typing import List, Optional

if sys.version_info >= (3, 8):
    from typing import TypedDict
else:
    from typing_extensions import TypedDict

from localstack.aws.api import RequestContext, ServiceException, ServiceRequest, handler

AccessLogEnabled = bool
AccessLogInterval = int
AccessLogPrefix = str
AccessPointName = str
AccessPointPort = int
AdditionalAttributeKey = str
AdditionalAttributeValue = str
AttributeName = str
AttributeType = str
AttributeValue = str
AvailabilityZone = str
Cardinality = str
ConnectionDrainingEnabled = bool
ConnectionDrainingTimeout = int
CookieName = str
CrossZoneLoadBalancingEnabled = bool
DNSName = str
DefaultValue = str
Description = str
EndPointPort = int
HealthCheckInterval = int
HealthCheckTarget = str
HealthCheckTimeout = int
HealthyThreshold = int
IdleTimeout = int
InstanceId = str
InstancePort = int
LoadBalancerScheme = str
Marker = str
Max = str
Name = str
PageSize = int
PolicyName = str
PolicyTypeName = str
Protocol = str
ReasonCode = str
S3BucketName = str
SSLCertificateId = str
SecurityGroupId = str
SecurityGroupName = str
SecurityGroupOwnerAlias = str
State = str
SubnetId = str
TagKey = str
TagValue = str
UnhealthyThreshold = int
VPCId = str


class AccessPointNotFoundException(ServiceException):
    """The specified load balancer does not exist."""


class CertificateNotFoundException(ServiceException):
    """The specified ARN does not refer to a valid SSL certificate in AWS
    Identity and Access Management (IAM) or AWS Certificate Manager (ACM).
    Note that if you recently uploaded the certificate to IAM, this error
    might indicate that the certificate is not fully available yet.
    """


class DependencyThrottleException(ServiceException):
    """A request made by Elastic Load Balancing to another service exceeds the
    maximum request rate permitted for your account.
    """


class DuplicateAccessPointNameException(ServiceException):
    """The specified load balancer name already exists for this account."""


class DuplicateListenerException(ServiceException):
    """A listener already exists for the specified load balancer name and port,
    but with a different instance port, protocol, or SSL certificate.
    """


class DuplicatePolicyNameException(ServiceException):
    """A policy with the specified name already exists for this load balancer."""


class DuplicateTagKeysException(ServiceException):
    """A tag key was specified more than once."""


class InvalidConfigurationRequestException(ServiceException):
    """The requested configuration change is not valid."""


class InvalidEndPointException(ServiceException):
    """The specified endpoint is not valid."""


class InvalidSchemeException(ServiceException):
    """The specified value for the schema is not valid. You can only specify a
    scheme for load balancers in a VPC.
    """


class InvalidSecurityGroupException(ServiceException):
    """One or more of the specified security groups do not exist."""


class InvalidSubnetException(ServiceException):
    """The specified VPC has no associated Internet gateway."""


class ListenerNotFoundException(ServiceException):
    """The load balancer does not have a listener configured at the specified
    port.
    """


class LoadBalancerAttributeNotFoundException(ServiceException):
    """The specified load balancer attribute does not exist."""


class OperationNotPermittedException(ServiceException):
    """This operation is not allowed."""


class PolicyNotFoundException(ServiceException):
    """One or more of the specified policies do not exist."""


class PolicyTypeNotFoundException(ServiceException):
    """One or more of the specified policy types do not exist."""


class SubnetNotFoundException(ServiceException):
    """One or more of the specified subnets do not exist."""


class TooManyAccessPointsException(ServiceException):
    """The quota for the number of load balancers has been reached."""


class TooManyPoliciesException(ServiceException):
    """The quota for the number of policies for this load balancer has been
    reached.
    """


class TooManyTagsException(ServiceException):
    """The quota for the number of tags that can be assigned to a load balancer
    has been reached.
    """


class UnsupportedProtocolException(ServiceException):
    """The specified protocol or signature version is not supported."""


class AccessLog(TypedDict, total=False):
    """Information about the ``AccessLog`` attribute."""

    Enabled: AccessLogEnabled
    S3BucketName: Optional[S3BucketName]
    EmitInterval: Optional[AccessLogInterval]
    S3BucketPrefix: Optional[AccessLogPrefix]


AvailabilityZones = List[AvailabilityZone]


class AddAvailabilityZonesInput(ServiceRequest):
    """Contains the parameters for EnableAvailabilityZonesForLoadBalancer."""

    LoadBalancerName: AccessPointName
    AvailabilityZones: AvailabilityZones


class AddAvailabilityZonesOutput(TypedDict, total=False):
    """Contains the output of EnableAvailabilityZonesForLoadBalancer."""

    AvailabilityZones: Optional[AvailabilityZones]


class Tag(TypedDict, total=False):
    """Information about a tag."""

    Key: TagKey
    Value: Optional[TagValue]


TagList = List[Tag]
LoadBalancerNames = List[AccessPointName]


class AddTagsInput(ServiceRequest):
    """Contains the parameters for AddTags."""

    LoadBalancerNames: LoadBalancerNames
    Tags: TagList


class AddTagsOutput(TypedDict, total=False):
    """Contains the output of AddTags."""


class AdditionalAttribute(TypedDict, total=False):
    """Information about additional load balancer attributes."""

    Key: Optional[AdditionalAttributeKey]
    Value: Optional[AdditionalAttributeValue]


AdditionalAttributes = List[AdditionalAttribute]


class AppCookieStickinessPolicy(TypedDict, total=False):
    """Information about a policy for application-controlled session
    stickiness.
    """

    PolicyName: Optional[PolicyName]
    CookieName: Optional[CookieName]


AppCookieStickinessPolicies = List[AppCookieStickinessPolicy]
SecurityGroups = List[SecurityGroupId]


class ApplySecurityGroupsToLoadBalancerInput(ServiceRequest):
    """Contains the parameters for ApplySecurityGroupsToLoadBalancer."""

    LoadBalancerName: AccessPointName
    SecurityGroups: SecurityGroups


class ApplySecurityGroupsToLoadBalancerOutput(TypedDict, total=False):
    """Contains the output of ApplySecurityGroupsToLoadBalancer."""

    SecurityGroups: Optional[SecurityGroups]


Subnets = List[SubnetId]


class AttachLoadBalancerToSubnetsInput(ServiceRequest):
    """Contains the parameters for AttachLoaBalancerToSubnets."""

    LoadBalancerName: AccessPointName
    Subnets: Subnets


class AttachLoadBalancerToSubnetsOutput(TypedDict, total=False):
    """Contains the output of AttachLoadBalancerToSubnets."""

    Subnets: Optional[Subnets]


PolicyNames = List[PolicyName]


class BackendServerDescription(TypedDict, total=False):
    """Information about the configuration of an EC2 instance."""

    InstancePort: Optional[InstancePort]
    PolicyNames: Optional[PolicyNames]


BackendServerDescriptions = List[BackendServerDescription]


class HealthCheck(TypedDict, total=False):
    """Information about a health check."""

    Target: HealthCheckTarget
    Interval: HealthCheckInterval
    Timeout: HealthCheckTimeout
    UnhealthyThreshold: UnhealthyThreshold
    HealthyThreshold: HealthyThreshold


class ConfigureHealthCheckInput(ServiceRequest):
    """Contains the parameters for ConfigureHealthCheck."""

    LoadBalancerName: AccessPointName
    HealthCheck: HealthCheck


class ConfigureHealthCheckOutput(TypedDict, total=False):
    """Contains the output of ConfigureHealthCheck."""

    HealthCheck: Optional[HealthCheck]


class ConnectionDraining(TypedDict, total=False):
    """Information about the ``ConnectionDraining`` attribute."""

    Enabled: ConnectionDrainingEnabled
    Timeout: Optional[ConnectionDrainingTimeout]


class ConnectionSettings(TypedDict, total=False):
    """Information about the ``ConnectionSettings`` attribute."""

    IdleTimeout: IdleTimeout


CookieExpirationPeriod = int


class Listener(TypedDict, total=False):
    """Information about a listener.

    For information about the protocols and the ports supported by Elastic
    Load Balancing, see `Listeners for Your Classic Load
    Balancer <https://docs.aws.amazon.com/elasticloadbalancing/latest/classic/elb-listener-config.html>`__
    in the *Classic Load Balancers Guide*.
    """

    Protocol: Protocol
    LoadBalancerPort: AccessPointPort
    InstanceProtocol: Optional[Protocol]
    InstancePort: InstancePort
    SSLCertificateId: Optional[SSLCertificateId]


Listeners = List[Listener]


class CreateAccessPointInput(ServiceRequest):
    """Contains the parameters for CreateLoadBalancer."""

    LoadBalancerName: AccessPointName
    Listeners: Listeners
    AvailabilityZones: Optional[AvailabilityZones]
    Subnets: Optional[Subnets]
    SecurityGroups: Optional[SecurityGroups]
    Scheme: Optional[LoadBalancerScheme]
    Tags: Optional[TagList]


class CreateAccessPointOutput(TypedDict, total=False):
    """Contains the output for CreateLoadBalancer."""

    DNSName: Optional[DNSName]


class CreateAppCookieStickinessPolicyInput(ServiceRequest):
    """Contains the parameters for CreateAppCookieStickinessPolicy."""

    LoadBalancerName: AccessPointName
    PolicyName: PolicyName
    CookieName: CookieName


class CreateAppCookieStickinessPolicyOutput(TypedDict, total=False):
    """Contains the output for CreateAppCookieStickinessPolicy."""


class CreateLBCookieStickinessPolicyInput(ServiceRequest):
    """Contains the parameters for CreateLBCookieStickinessPolicy."""

    LoadBalancerName: AccessPointName
    PolicyName: PolicyName
    CookieExpirationPeriod: Optional[CookieExpirationPeriod]


class CreateLBCookieStickinessPolicyOutput(TypedDict, total=False):
    """Contains the output for CreateLBCookieStickinessPolicy."""


class CreateLoadBalancerListenerInput(ServiceRequest):
    """Contains the parameters for CreateLoadBalancerListeners."""

    LoadBalancerName: AccessPointName
    Listeners: Listeners


class CreateLoadBalancerListenerOutput(TypedDict, total=False):
    """Contains the parameters for CreateLoadBalancerListener."""


class PolicyAttribute(TypedDict, total=False):
    """Information about a policy attribute."""

    AttributeName: Optional[AttributeName]
    AttributeValue: Optional[AttributeValue]


PolicyAttributes = List[PolicyAttribute]


class CreateLoadBalancerPolicyInput(ServiceRequest):
    """Contains the parameters for CreateLoadBalancerPolicy."""

    LoadBalancerName: AccessPointName
    PolicyName: PolicyName
    PolicyTypeName: PolicyTypeName
    PolicyAttributes: Optional[PolicyAttributes]


class CreateLoadBalancerPolicyOutput(TypedDict, total=False):
    """Contains the output of CreateLoadBalancerPolicy."""


CreatedTime = datetime


class CrossZoneLoadBalancing(TypedDict, total=False):
    """Information about the ``CrossZoneLoadBalancing`` attribute."""

    Enabled: CrossZoneLoadBalancingEnabled


class DeleteAccessPointInput(ServiceRequest):
    """Contains the parameters for DeleteLoadBalancer."""

    LoadBalancerName: AccessPointName


class DeleteAccessPointOutput(TypedDict, total=False):
    """Contains the output of DeleteLoadBalancer."""


Ports = List[AccessPointPort]


class DeleteLoadBalancerListenerInput(ServiceRequest):
    """Contains the parameters for DeleteLoadBalancerListeners."""

    LoadBalancerName: AccessPointName
    LoadBalancerPorts: Ports


class DeleteLoadBalancerListenerOutput(TypedDict, total=False):
    """Contains the output of DeleteLoadBalancerListeners."""


class DeleteLoadBalancerPolicyInput(ServiceRequest):
    """Contains the parameters for DeleteLoadBalancerPolicy."""

    LoadBalancerName: AccessPointName
    PolicyName: PolicyName


class DeleteLoadBalancerPolicyOutput(TypedDict, total=False):
    """Contains the output of DeleteLoadBalancerPolicy."""


class Instance(TypedDict, total=False):
    """The ID of an EC2 instance."""

    InstanceId: Optional[InstanceId]


Instances = List[Instance]


class DeregisterEndPointsInput(ServiceRequest):
    """Contains the parameters for DeregisterInstancesFromLoadBalancer."""

    LoadBalancerName: AccessPointName
    Instances: Instances


class DeregisterEndPointsOutput(TypedDict, total=False):
    """Contains the output of DeregisterInstancesFromLoadBalancer."""

    Instances: Optional[Instances]


class DescribeAccessPointsInput(ServiceRequest):
    """Contains the parameters for DescribeLoadBalancers."""

    LoadBalancerNames: Optional[LoadBalancerNames]
    Marker: Optional[Marker]
    PageSize: Optional[PageSize]


class SourceSecurityGroup(TypedDict, total=False):
    """Information about a source security group."""

    OwnerAlias: Optional[SecurityGroupOwnerAlias]
    GroupName: Optional[SecurityGroupName]


class LBCookieStickinessPolicy(TypedDict, total=False):
    """Information about a policy for duration-based session stickiness."""

    PolicyName: Optional[PolicyName]
    CookieExpirationPeriod: Optional[CookieExpirationPeriod]


LBCookieStickinessPolicies = List[LBCookieStickinessPolicy]


class Policies(TypedDict, total=False):
    """The policies for a load balancer."""

    AppCookieStickinessPolicies: Optional[AppCookieStickinessPolicies]
    LBCookieStickinessPolicies: Optional[LBCookieStickinessPolicies]
    OtherPolicies: Optional[PolicyNames]


class ListenerDescription(TypedDict, total=False):
    """The policies enabled for a listener."""

    Listener: Optional[Listener]
    PolicyNames: Optional[PolicyNames]


ListenerDescriptions = List[ListenerDescription]


class LoadBalancerDescription(TypedDict, total=False):
    """Information about a load balancer."""

    LoadBalancerName: Optional[AccessPointName]
    DNSName: Optional[DNSName]
    CanonicalHostedZoneName: Optional[DNSName]
    CanonicalHostedZoneNameID: Optional[DNSName]
    ListenerDescriptions: Optional[ListenerDescriptions]
    Policies: Optional[Policies]
    BackendServerDescriptions: Optional[BackendServerDescriptions]
    AvailabilityZones: Optional[AvailabilityZones]
    Subnets: Optional[Subnets]
    VPCId: Optional[VPCId]
    Instances: Optional[Instances]
    HealthCheck: Optional[HealthCheck]
    SourceSecurityGroup: Optional[SourceSecurityGroup]
    SecurityGroups: Optional[SecurityGroups]
    CreatedTime: Optional[CreatedTime]
    Scheme: Optional[LoadBalancerScheme]


LoadBalancerDescriptions = List[LoadBalancerDescription]


class DescribeAccessPointsOutput(TypedDict, total=False):
    """Contains the parameters for DescribeLoadBalancers."""

    LoadBalancerDescriptions: Optional[LoadBalancerDescriptions]
    NextMarker: Optional[Marker]


class DescribeAccountLimitsInput(ServiceRequest):
    Marker: Optional[Marker]
    PageSize: Optional[PageSize]


class Limit(TypedDict, total=False):
    """Information about an Elastic Load Balancing resource limit for your AWS
    account.
    """

    Name: Optional[Name]
    Max: Optional[Max]


Limits = List[Limit]


class DescribeAccountLimitsOutput(TypedDict, total=False):
    Limits: Optional[Limits]
    NextMarker: Optional[Marker]


class DescribeEndPointStateInput(ServiceRequest):
    """Contains the parameters for DescribeInstanceHealth."""

    LoadBalancerName: AccessPointName
    Instances: Optional[Instances]


class InstanceState(TypedDict, total=False):
    """Information about the state of an EC2 instance."""

    InstanceId: Optional[InstanceId]
    State: Optional[State]
    ReasonCode: Optional[ReasonCode]
    Description: Optional[Description]


InstanceStates = List[InstanceState]


class DescribeEndPointStateOutput(TypedDict, total=False):
    """Contains the output for DescribeInstanceHealth."""

    InstanceStates: Optional[InstanceStates]


class DescribeLoadBalancerAttributesInput(ServiceRequest):
    """Contains the parameters for DescribeLoadBalancerAttributes."""

    LoadBalancerName: AccessPointName


class LoadBalancerAttributes(TypedDict, total=False):
    """The attributes for a load balancer."""

    CrossZoneLoadBalancing: Optional[CrossZoneLoadBalancing]
    AccessLog: Optional[AccessLog]
    ConnectionDraining: Optional[ConnectionDraining]
    ConnectionSettings: Optional[ConnectionSettings]
    AdditionalAttributes: Optional[AdditionalAttributes]


class DescribeLoadBalancerAttributesOutput(TypedDict, total=False):
    """Contains the output of DescribeLoadBalancerAttributes."""

    LoadBalancerAttributes: Optional[LoadBalancerAttributes]


class DescribeLoadBalancerPoliciesInput(ServiceRequest):
    """Contains the parameters for DescribeLoadBalancerPolicies."""

    LoadBalancerName: Optional[AccessPointName]
    PolicyNames: Optional[PolicyNames]


class PolicyAttributeDescription(TypedDict, total=False):
    """Information about a policy attribute."""

    AttributeName: Optional[AttributeName]
    AttributeValue: Optional[AttributeValue]


PolicyAttributeDescriptions = List[PolicyAttributeDescription]


class PolicyDescription(TypedDict, total=False):
    """Information about a policy."""

    PolicyName: Optional[PolicyName]
    PolicyTypeName: Optional[PolicyTypeName]
    PolicyAttributeDescriptions: Optional[PolicyAttributeDescriptions]


PolicyDescriptions = List[PolicyDescription]


class DescribeLoadBalancerPoliciesOutput(TypedDict, total=False):
    """Contains the output of DescribeLoadBalancerPolicies."""

    PolicyDescriptions: Optional[PolicyDescriptions]


PolicyTypeNames = List[PolicyTypeName]


class DescribeLoadBalancerPolicyTypesInput(ServiceRequest):
    """Contains the parameters for DescribeLoadBalancerPolicyTypes."""

    PolicyTypeNames: Optional[PolicyTypeNames]


class PolicyAttributeTypeDescription(TypedDict, total=False):
    """Information about a policy attribute type."""

    AttributeName: Optional[AttributeName]
    AttributeType: Optional[AttributeType]
    Description: Optional[Description]
    DefaultValue: Optional[DefaultValue]
    Cardinality: Optional[Cardinality]


PolicyAttributeTypeDescriptions = List[PolicyAttributeTypeDescription]


class PolicyTypeDescription(TypedDict, total=False):
    """Information about a policy type."""

    PolicyTypeName: Optional[PolicyTypeName]
    Description: Optional[Description]
    PolicyAttributeTypeDescriptions: Optional[PolicyAttributeTypeDescriptions]


PolicyTypeDescriptions = List[PolicyTypeDescription]


class DescribeLoadBalancerPolicyTypesOutput(TypedDict, total=False):
    """Contains the output of DescribeLoadBalancerPolicyTypes."""

    PolicyTypeDescriptions: Optional[PolicyTypeDescriptions]


LoadBalancerNamesMax20 = List[AccessPointName]


class DescribeTagsInput(ServiceRequest):
    """Contains the parameters for DescribeTags."""

    LoadBalancerNames: LoadBalancerNamesMax20


class TagDescription(TypedDict, total=False):
    """The tags associated with a load balancer."""

    LoadBalancerName: Optional[AccessPointName]
    Tags: Optional[TagList]


TagDescriptions = List[TagDescription]


class DescribeTagsOutput(TypedDict, total=False):
    """Contains the output for DescribeTags."""

    TagDescriptions: Optional[TagDescriptions]


class DetachLoadBalancerFromSubnetsInput(ServiceRequest):
    """Contains the parameters for DetachLoadBalancerFromSubnets."""

    LoadBalancerName: AccessPointName
    Subnets: Subnets


class DetachLoadBalancerFromSubnetsOutput(TypedDict, total=False):
    """Contains the output of DetachLoadBalancerFromSubnets."""

    Subnets: Optional[Subnets]


class ModifyLoadBalancerAttributesInput(ServiceRequest):
    """Contains the parameters for ModifyLoadBalancerAttributes."""

    LoadBalancerName: AccessPointName
    LoadBalancerAttributes: LoadBalancerAttributes


class ModifyLoadBalancerAttributesOutput(TypedDict, total=False):
    """Contains the output of ModifyLoadBalancerAttributes."""

    LoadBalancerName: Optional[AccessPointName]
    LoadBalancerAttributes: Optional[LoadBalancerAttributes]


class RegisterEndPointsInput(ServiceRequest):
    """Contains the parameters for RegisterInstancesWithLoadBalancer."""

    LoadBalancerName: AccessPointName
    Instances: Instances


class RegisterEndPointsOutput(TypedDict, total=False):
    """Contains the output of RegisterInstancesWithLoadBalancer."""

    Instances: Optional[Instances]


class RemoveAvailabilityZonesInput(ServiceRequest):
    """Contains the parameters for DisableAvailabilityZonesForLoadBalancer."""

    LoadBalancerName: AccessPointName
    AvailabilityZones: AvailabilityZones


class RemoveAvailabilityZonesOutput(TypedDict, total=False):
    """Contains the output for DisableAvailabilityZonesForLoadBalancer."""

    AvailabilityZones: Optional[AvailabilityZones]


class TagKeyOnly(TypedDict, total=False):
    """The key of a tag."""

    Key: Optional[TagKey]


TagKeyList = List[TagKeyOnly]


class RemoveTagsInput(ServiceRequest):
    """Contains the parameters for RemoveTags."""

    LoadBalancerNames: LoadBalancerNames
    Tags: TagKeyList


class RemoveTagsOutput(TypedDict, total=False):
    """Contains the output of RemoveTags."""


class SetLoadBalancerListenerSSLCertificateInput(ServiceRequest):
    """Contains the parameters for SetLoadBalancerListenerSSLCertificate."""

    LoadBalancerName: AccessPointName
    LoadBalancerPort: AccessPointPort
    SSLCertificateId: SSLCertificateId


class SetLoadBalancerListenerSSLCertificateOutput(TypedDict, total=False):
    """Contains the output of SetLoadBalancerListenerSSLCertificate."""


class SetLoadBalancerPoliciesForBackendServerInput(ServiceRequest):
    """Contains the parameters for SetLoadBalancerPoliciesForBackendServer."""

    LoadBalancerName: AccessPointName
    InstancePort: EndPointPort
    PolicyNames: PolicyNames


class SetLoadBalancerPoliciesForBackendServerOutput(TypedDict, total=False):
    """Contains the output of SetLoadBalancerPoliciesForBackendServer."""


class SetLoadBalancerPoliciesOfListenerInput(ServiceRequest):
    """Contains the parameters for SetLoadBalancePoliciesOfListener."""

    LoadBalancerName: AccessPointName
    LoadBalancerPort: AccessPointPort
    PolicyNames: PolicyNames


class SetLoadBalancerPoliciesOfListenerOutput(TypedDict, total=False):
    """Contains the output of SetLoadBalancePoliciesOfListener."""


class ElbApi:

    service = "elb"
    version = "2012-06-01"

    @handler("AddTags")
    def add_tags(
        self, context: RequestContext, load_balancer_names: LoadBalancerNames, tags: TagList
    ) -> AddTagsOutput:
        """Adds the specified tags to the specified load balancer. Each load
        balancer can have a maximum of 10 tags.

        Each tag consists of a key and an optional value. If a tag with the same
        key is already associated with the load balancer, ``AddTags`` updates
        its value.

        For more information, see `Tag Your Classic Load
        Balancer <https://docs.aws.amazon.com/elasticloadbalancing/latest/classic/add-remove-tags.html>`__
        in the *Classic Load Balancers Guide*.

        :param load_balancer_names: The name of the load balancer.
        :param tags: The tags.
        :returns: AddTagsOutput
        :raises AccessPointNotFoundException:
        :raises TooManyTagsException:
        :raises DuplicateTagKeysException:
        """
        raise NotImplementedError

    @handler("ApplySecurityGroupsToLoadBalancer")
    def apply_security_groups_to_load_balancer(
        self,
        context: RequestContext,
        load_balancer_name: AccessPointName,
        security_groups: SecurityGroups,
    ) -> ApplySecurityGroupsToLoadBalancerOutput:
        """Associates one or more security groups with your load balancer in a
        virtual private cloud (VPC). The specified security groups override the
        previously associated security groups.

        For more information, see `Security Groups for Load Balancers in a
        VPC <https://docs.aws.amazon.com/elasticloadbalancing/latest/classic/elb-security-groups.html#elb-vpc-security-groups>`__
        in the *Classic Load Balancers Guide*.

        :param load_balancer_name: The name of the load balancer.
        :param security_groups: The IDs of the security groups to associate with the load balancer.
        :returns: ApplySecurityGroupsToLoadBalancerOutput
        :raises AccessPointNotFoundException:
        :raises InvalidConfigurationRequestException:
        :raises InvalidSecurityGroupException:
        """
        raise NotImplementedError

    @handler("AttachLoadBalancerToSubnets")
    def attach_load_balancer_to_subnets(
        self, context: RequestContext, load_balancer_name: AccessPointName, subnets: Subnets
    ) -> AttachLoadBalancerToSubnetsOutput:
        """Adds one or more subnets to the set of configured subnets for the
        specified load balancer.

        The load balancer evenly distributes requests across all registered
        subnets. For more information, see `Add or Remove Subnets for Your Load
        Balancer in a
        VPC <https://docs.aws.amazon.com/elasticloadbalancing/latest/classic/elb-manage-subnets.html>`__
        in the *Classic Load Balancers Guide*.

        :param load_balancer_name: The name of the load balancer.
        :param subnets: The IDs of the subnets to add.
        :returns: AttachLoadBalancerToSubnetsOutput
        :raises AccessPointNotFoundException:
        :raises InvalidConfigurationRequestException:
        :raises SubnetNotFoundException:
        :raises InvalidSubnetException:
        """
        raise NotImplementedError

    @handler("ConfigureHealthCheck")
    def configure_health_check(
        self,
        context: RequestContext,
        load_balancer_name: AccessPointName,
        health_check: HealthCheck,
    ) -> ConfigureHealthCheckOutput:
        """Specifies the health check settings to use when evaluating the health
        state of your EC2 instances.

        For more information, see `Configure Health Checks for Your Load
        Balancer <https://docs.aws.amazon.com/elasticloadbalancing/latest/classic/elb-healthchecks.html>`__
        in the *Classic Load Balancers Guide*.

        :param load_balancer_name: The name of the load balancer.
        :param health_check: The configuration information.
        :returns: ConfigureHealthCheckOutput
        :raises AccessPointNotFoundException:
        """
        raise NotImplementedError

    @handler("CreateAppCookieStickinessPolicy")
    def create_app_cookie_stickiness_policy(
        self,
        context: RequestContext,
        load_balancer_name: AccessPointName,
        policy_name: PolicyName,
        cookie_name: CookieName,
    ) -> CreateAppCookieStickinessPolicyOutput:
        """Generates a stickiness policy with sticky session lifetimes that follow
        that of an application-generated cookie. This policy can be associated
        only with HTTP/HTTPS listeners.

        This policy is similar to the policy created by
        CreateLBCookieStickinessPolicy, except that the lifetime of the special
        Elastic Load Balancing cookie, ``AWSELB``, follows the lifetime of the
        application-generated cookie specified in the policy configuration. The
        load balancer only inserts a new stickiness cookie when the application
        response includes a new application cookie.

        If the application cookie is explicitly removed or expires, the session
        stops being sticky until a new application cookie is issued.

        For more information, see `Application-Controlled Session
        Stickiness <https://docs.aws.amazon.com/elasticloadbalancing/latest/classic/elb-sticky-sessions.html#enable-sticky-sessions-application>`__
        in the *Classic Load Balancers Guide*.

        :param load_balancer_name: The name of the load balancer.
        :param policy_name: The name of the policy being created.
        :param cookie_name: The name of the application cookie used for stickiness.
        :returns: CreateAppCookieStickinessPolicyOutput
        :raises AccessPointNotFoundException:
        :raises DuplicatePolicyNameException:
        :raises TooManyPoliciesException:
        :raises InvalidConfigurationRequestException:
        """
        raise NotImplementedError

    @handler("CreateLBCookieStickinessPolicy")
    def create_lb_cookie_stickiness_policy(
        self,
        context: RequestContext,
        load_balancer_name: AccessPointName,
        policy_name: PolicyName,
        cookie_expiration_period: CookieExpirationPeriod = None,
    ) -> CreateLBCookieStickinessPolicyOutput:
        """Generates a stickiness policy with sticky session lifetimes controlled
        by the lifetime of the browser (user-agent) or a specified expiration
        period. This policy can be associated only with HTTP/HTTPS listeners.

        When a load balancer implements this policy, the load balancer uses a
        special cookie to track the instance for each request. When the load
        balancer receives a request, it first checks to see if this cookie is
        present in the request. If so, the load balancer sends the request to
        the application server specified in the cookie. If not, the load
        balancer sends the request to a server that is chosen based on the
        existing load-balancing algorithm.

        A cookie is inserted into the response for binding subsequent requests
        from the same user to that server. The validity of the cookie is based
        on the cookie expiration time, which is specified in the policy
        configuration.

        For more information, see `Duration-Based Session
        Stickiness <https://docs.aws.amazon.com/elasticloadbalancing/latest/classic/elb-sticky-sessions.html#enable-sticky-sessions-duration>`__
        in the *Classic Load Balancers Guide*.

        :param load_balancer_name: The name of the load balancer.
        :param policy_name: The name of the policy being created.
        :param cookie_expiration_period: The time period, in seconds, after which the cookie should be considered
        stale.
        :returns: CreateLBCookieStickinessPolicyOutput
        :raises AccessPointNotFoundException:
        :raises DuplicatePolicyNameException:
        :raises TooManyPoliciesException:
        :raises InvalidConfigurationRequestException:
        """
        raise NotImplementedError

    @handler("CreateLoadBalancer")
    def create_load_balancer(
        self,
        context: RequestContext,
        load_balancer_name: AccessPointName,
        listeners: Listeners,
        availability_zones: AvailabilityZones = None,
        subnets: Subnets = None,
        security_groups: SecurityGroups = None,
        scheme: LoadBalancerScheme = None,
        tags: TagList = None,
    ) -> CreateAccessPointOutput:
        """Creates a Classic Load Balancer.

        You can add listeners, security groups, subnets, and tags when you
        create your load balancer, or you can add them later using
        CreateLoadBalancerListeners, ApplySecurityGroupsToLoadBalancer,
        AttachLoadBalancerToSubnets, and AddTags.

        To describe your current load balancers, see DescribeLoadBalancers. When
        you are finished with a load balancer, you can delete it using
        DeleteLoadBalancer.

        You can create up to 20 load balancers per region per account. You can
        request an increase for the number of load balancers for your account.
        For more information, see `Limits for Your Classic Load
        Balancer <https://docs.aws.amazon.com/elasticloadbalancing/latest/classic/elb-limits.html>`__
        in the *Classic Load Balancers Guide*.

        :param load_balancer_name: The name of the load balancer.
        :param listeners: The listeners.
        :param availability_zones: One or more Availability Zones from the same region as the load
        balancer.
        :param subnets: The IDs of the subnets in your VPC to attach to the load balancer.
        :param security_groups: The IDs of the security groups to assign to the load balancer.
        :param scheme: The type of a load balancer.
        :param tags: A list of tags to assign to the load balancer.
        :returns: CreateAccessPointOutput
        :raises DuplicateAccessPointNameException:
        :raises TooManyAccessPointsException:
        :raises CertificateNotFoundException:
        :raises InvalidConfigurationRequestException:
        :raises SubnetNotFoundException:
        :raises InvalidSubnetException:
        :raises InvalidSecurityGroupException:
        :raises InvalidSchemeException:
        :raises TooManyTagsException:
        :raises DuplicateTagKeysException:
        :raises UnsupportedProtocolException:
        :raises OperationNotPermittedException:
        """
        raise NotImplementedError

    @handler("CreateLoadBalancerListeners")
    def create_load_balancer_listeners(
        self, context: RequestContext, load_balancer_name: AccessPointName, listeners: Listeners
    ) -> CreateLoadBalancerListenerOutput:
        """Creates one or more listeners for the specified load balancer. If a
        listener with the specified port does not already exist, it is created;
        otherwise, the properties of the new listener must match the properties
        of the existing listener.

        For more information, see `Listeners for Your Classic Load
        Balancer <https://docs.aws.amazon.com/elasticloadbalancing/latest/classic/elb-listener-config.html>`__
        in the *Classic Load Balancers Guide*.

        :param load_balancer_name: The name of the load balancer.
        :param listeners: The listeners.
        :returns: CreateLoadBalancerListenerOutput
        :raises AccessPointNotFoundException:
        :raises DuplicateListenerException:
        :raises CertificateNotFoundException:
        :raises InvalidConfigurationRequestException:
        :raises UnsupportedProtocolException:
        """
        raise NotImplementedError

    @handler("CreateLoadBalancerPolicy")
    def create_load_balancer_policy(
        self,
        context: RequestContext,
        load_balancer_name: AccessPointName,
        policy_name: PolicyName,
        policy_type_name: PolicyTypeName,
        policy_attributes: PolicyAttributes = None,
    ) -> CreateLoadBalancerPolicyOutput:
        """Creates a policy with the specified attributes for the specified load
        balancer.

        Policies are settings that are saved for your load balancer and that can
        be applied to the listener or the application server, depending on the
        policy type.

        :param load_balancer_name: The name of the load balancer.
        :param policy_name: The name of the load balancer policy to be created.
        :param policy_type_name: The name of the base policy type.
        :param policy_attributes: The policy attributes.
        :returns: CreateLoadBalancerPolicyOutput
        :raises AccessPointNotFoundException:
        :raises PolicyTypeNotFoundException:
        :raises DuplicatePolicyNameException:
        :raises TooManyPoliciesException:
        :raises InvalidConfigurationRequestException:
        """
        raise NotImplementedError

    @handler("DeleteLoadBalancer")
    def delete_load_balancer(
        self, context: RequestContext, load_balancer_name: AccessPointName
    ) -> DeleteAccessPointOutput:
        """Deletes the specified load balancer.

        If you are attempting to recreate a load balancer, you must reconfigure
        all settings. The DNS name associated with a deleted load balancer are
        no longer usable. The name and associated DNS record of the deleted load
        balancer no longer exist and traffic sent to any of its IP addresses is
        no longer delivered to your instances.

        If the load balancer does not exist or has already been deleted, the
        call to ``DeleteLoadBalancer`` still succeeds.

        :param load_balancer_name: The name of the load balancer.
        :returns: DeleteAccessPointOutput
        """
        raise NotImplementedError

    @handler("DeleteLoadBalancerListeners")
    def delete_load_balancer_listeners(
        self,
        context: RequestContext,
        load_balancer_name: AccessPointName,
        load_balancer_ports: Ports,
    ) -> DeleteLoadBalancerListenerOutput:
        """Deletes the specified listeners from the specified load balancer.

        :param load_balancer_name: The name of the load balancer.
        :param load_balancer_ports: The client port numbers of the listeners.
        :returns: DeleteLoadBalancerListenerOutput
        :raises AccessPointNotFoundException:
        """
        raise NotImplementedError

    @handler("DeleteLoadBalancerPolicy")
    def delete_load_balancer_policy(
        self, context: RequestContext, load_balancer_name: AccessPointName, policy_name: PolicyName
    ) -> DeleteLoadBalancerPolicyOutput:
        """Deletes the specified policy from the specified load balancer. This
        policy must not be enabled for any listeners.

        :param load_balancer_name: The name of the load balancer.
        :param policy_name: The name of the policy.
        :returns: DeleteLoadBalancerPolicyOutput
        :raises AccessPointNotFoundException:
        :raises InvalidConfigurationRequestException:
        """
        raise NotImplementedError

    @handler("DeregisterInstancesFromLoadBalancer")
    def deregister_instances_from_load_balancer(
        self, context: RequestContext, load_balancer_name: AccessPointName, instances: Instances
    ) -> DeregisterEndPointsOutput:
        """Deregisters the specified instances from the specified load balancer.
        After the instance is deregistered, it no longer receives traffic from
        the load balancer.

        You can use DescribeLoadBalancers to verify that the instance is
        deregistered from the load balancer.

        For more information, see `Register or De-Register EC2
        Instances <https://docs.aws.amazon.com/elasticloadbalancing/latest/classic/elb-deregister-register-instances.html>`__
        in the *Classic Load Balancers Guide*.

        :param load_balancer_name: The name of the load balancer.
        :param instances: The IDs of the instances.
        :returns: DeregisterEndPointsOutput
        :raises AccessPointNotFoundException:
        :raises InvalidEndPointException:
        """
        raise NotImplementedError

    @handler("DescribeAccountLimits")
    def describe_account_limits(
        self, context: RequestContext, marker: Marker = None, page_size: PageSize = None
    ) -> DescribeAccountLimitsOutput:
        """Describes the current Elastic Load Balancing resource limits for your
        AWS account.

        For more information, see `Limits for Your Classic Load
        Balancer <https://docs.aws.amazon.com/elasticloadbalancing/latest/classic/elb-limits.html>`__
        in the *Classic Load Balancers Guide*.

        :param marker: The marker for the next set of results.
        :param page_size: The maximum number of results to return with this call.
        :returns: DescribeAccountLimitsOutput
        """
        raise NotImplementedError

    @handler("DescribeInstanceHealth")
    def describe_instance_health(
        self,
        context: RequestContext,
        load_balancer_name: AccessPointName,
        instances: Instances = None,
    ) -> DescribeEndPointStateOutput:
        """Describes the state of the specified instances with respect to the
        specified load balancer. If no instances are specified, the call
        describes the state of all instances that are currently registered with
        the load balancer. If instances are specified, their state is returned
        even if they are no longer registered with the load balancer. The state
        of terminated instances is not returned.

        :param load_balancer_name: The name of the load balancer.
        :param instances: The IDs of the instances.
        :returns: DescribeEndPointStateOutput
        :raises AccessPointNotFoundException:
        :raises InvalidEndPointException:
        """
        raise NotImplementedError

    @handler("DescribeLoadBalancerAttributes")
    def describe_load_balancer_attributes(
        self, context: RequestContext, load_balancer_name: AccessPointName
    ) -> DescribeLoadBalancerAttributesOutput:
        """Describes the attributes for the specified load balancer.

        :param load_balancer_name: The name of the load balancer.
        :returns: DescribeLoadBalancerAttributesOutput
        :raises AccessPointNotFoundException:
        :raises LoadBalancerAttributeNotFoundException:
        """
        raise NotImplementedError

    @handler("DescribeLoadBalancerPolicies")
    def describe_load_balancer_policies(
        self,
        context: RequestContext,
        load_balancer_name: AccessPointName = None,
        policy_names: PolicyNames = None,
    ) -> DescribeLoadBalancerPoliciesOutput:
        """Describes the specified policies.

        If you specify a load balancer name, the action returns the descriptions
        of all policies created for the load balancer. If you specify a policy
        name associated with your load balancer, the action returns the
        description of that policy. If you don't specify a load balancer name,
        the action returns descriptions of the specified sample policies, or
        descriptions of all sample policies. The names of the sample policies
        have the ``ELBSample-`` prefix.

        :param load_balancer_name: The name of the load balancer.
        :param policy_names: The names of the policies.
        :returns: DescribeLoadBalancerPoliciesOutput
        :raises AccessPointNotFoundException:
        :raises PolicyNotFoundException:
        """
        raise NotImplementedError

    @handler("DescribeLoadBalancerPolicyTypes")
    def describe_load_balancer_policy_types(
        self, context: RequestContext, policy_type_names: PolicyTypeNames = None
    ) -> DescribeLoadBalancerPolicyTypesOutput:
        """Describes the specified load balancer policy types or all load balancer
        policy types.

        The description of each type indicates how it can be used. For example,
        some policies can be used only with layer 7 listeners, some policies can
        be used only with layer 4 listeners, and some policies can be used only
        with your EC2 instances.

        You can use CreateLoadBalancerPolicy to create a policy configuration
        for any of these policy types. Then, depending on the policy type, use
        either SetLoadBalancerPoliciesOfListener or
        SetLoadBalancerPoliciesForBackendServer to set the policy.

        :param policy_type_names: The names of the policy types.
        :returns: DescribeLoadBalancerPolicyTypesOutput
        :raises PolicyTypeNotFoundException:
        """
        raise NotImplementedError

    @handler("DescribeLoadBalancers")
    def describe_load_balancers(
        self,
        context: RequestContext,
        load_balancer_names: LoadBalancerNames = None,
        marker: Marker = None,
        page_size: PageSize = None,
    ) -> DescribeAccessPointsOutput:
        """Describes the specified the load balancers. If no load balancers are
        specified, the call describes all of your load balancers.

        :param load_balancer_names: The names of the load balancers.
        :param marker: The marker for the next set of results.
        :param page_size: The maximum number of results to return with this call (a number from 1
        to 400).
        :returns: DescribeAccessPointsOutput
        :raises AccessPointNotFoundException:
        :raises DependencyThrottleException:
        """
        raise NotImplementedError

    @handler("DescribeTags")
    def describe_tags(
        self, context: RequestContext, load_balancer_names: LoadBalancerNamesMax20
    ) -> DescribeTagsOutput:
        """Describes the tags associated with the specified load balancers.

        :param load_balancer_names: The names of the load balancers.
        :returns: DescribeTagsOutput
        :raises AccessPointNotFoundException:
        """
        raise NotImplementedError

    @handler("DetachLoadBalancerFromSubnets")
    def detach_load_balancer_from_subnets(
        self, context: RequestContext, load_balancer_name: AccessPointName, subnets: Subnets
    ) -> DetachLoadBalancerFromSubnetsOutput:
        """Removes the specified subnets from the set of configured subnets for the
        load balancer.

        After a subnet is removed, all EC2 instances registered with the load
        balancer in the removed subnet go into the ``OutOfService`` state. Then,
        the load balancer balances the traffic among the remaining routable
        subnets.

        :param load_balancer_name: The name of the load balancer.
        :param subnets: The IDs of the subnets.
        :returns: DetachLoadBalancerFromSubnetsOutput
        :raises AccessPointNotFoundException:
        :raises InvalidConfigurationRequestException:
        """
        raise NotImplementedError

    @handler("DisableAvailabilityZonesForLoadBalancer")
    def disable_availability_zones_for_load_balancer(
        self,
        context: RequestContext,
        load_balancer_name: AccessPointName,
        availability_zones: AvailabilityZones,
    ) -> RemoveAvailabilityZonesOutput:
        """Removes the specified Availability Zones from the set of Availability
        Zones for the specified load balancer in EC2-Classic or a default VPC.

        For load balancers in a non-default VPC, use
        DetachLoadBalancerFromSubnets.

        There must be at least one Availability Zone registered with a load
        balancer at all times. After an Availability Zone is removed, all
        instances registered with the load balancer that are in the removed
        Availability Zone go into the ``OutOfService`` state. Then, the load
        balancer attempts to equally balance the traffic among its remaining
        Availability Zones.

        For more information, see `Add or Remove Availability
        Zones <https://docs.aws.amazon.com/elasticloadbalancing/latest/classic/enable-disable-az.html>`__
        in the *Classic Load Balancers Guide*.

        :param load_balancer_name: The name of the load balancer.
        :param availability_zones: The Availability Zones.
        :returns: RemoveAvailabilityZonesOutput
        :raises AccessPointNotFoundException:
        :raises InvalidConfigurationRequestException:
        """
        raise NotImplementedError

    @handler("EnableAvailabilityZonesForLoadBalancer")
    def enable_availability_zones_for_load_balancer(
        self,
        context: RequestContext,
        load_balancer_name: AccessPointName,
        availability_zones: AvailabilityZones,
    ) -> AddAvailabilityZonesOutput:
        """Adds the specified Availability Zones to the set of Availability Zones
        for the specified load balancer in EC2-Classic or a default VPC.

        For load balancers in a non-default VPC, use
        AttachLoadBalancerToSubnets.

        The load balancer evenly distributes requests across all its registered
        Availability Zones that contain instances. For more information, see
        `Add or Remove Availability
        Zones <https://docs.aws.amazon.com/elasticloadbalancing/latest/classic/enable-disable-az.html>`__
        in the *Classic Load Balancers Guide*.

        :param load_balancer_name: The name of the load balancer.
        :param availability_zones: The Availability Zones.
        :returns: AddAvailabilityZonesOutput
        :raises AccessPointNotFoundException:
        """
        raise NotImplementedError

    @handler("ModifyLoadBalancerAttributes")
    def modify_load_balancer_attributes(
        self,
        context: RequestContext,
        load_balancer_name: AccessPointName,
        load_balancer_attributes: LoadBalancerAttributes,
    ) -> ModifyLoadBalancerAttributesOutput:
        """Modifies the attributes of the specified load balancer.

        You can modify the load balancer attributes, such as ``AccessLogs``,
        ``ConnectionDraining``, and ``CrossZoneLoadBalancing`` by either
        enabling or disabling them. Or, you can modify the load balancer
        attribute ``ConnectionSettings`` by specifying an idle connection
        timeout value for your load balancer.

        For more information, see the following in the *Classic Load Balancers
        Guide*:

        -  `Cross-Zone Load
           Balancing <https://docs.aws.amazon.com/elasticloadbalancing/latest/classic/enable-disable-crosszone-lb.html>`__

        -  `Connection
           Draining <https://docs.aws.amazon.com/elasticloadbalancing/latest/classic/config-conn-drain.html>`__

        -  `Access
           Logs <https://docs.aws.amazon.com/elasticloadbalancing/latest/classic/access-log-collection.html>`__

        -  `Idle Connection
           Timeout <https://docs.aws.amazon.com/elasticloadbalancing/latest/classic/config-idle-timeout.html>`__

        :param load_balancer_name: The name of the load balancer.
        :param load_balancer_attributes: The attributes for the load balancer.
        :returns: ModifyLoadBalancerAttributesOutput
        :raises AccessPointNotFoundException:
        :raises LoadBalancerAttributeNotFoundException:
        :raises InvalidConfigurationRequestException:
        """
        raise NotImplementedError

    @handler("RegisterInstancesWithLoadBalancer")
    def register_instances_with_load_balancer(
        self, context: RequestContext, load_balancer_name: AccessPointName, instances: Instances
    ) -> RegisterEndPointsOutput:
        """Adds the specified instances to the specified load balancer.

        The instance must be a running instance in the same network as the load
        balancer (EC2-Classic or the same VPC). If you have EC2-Classic
        instances and a load balancer in a VPC with ClassicLink enabled, you can
        link the EC2-Classic instances to that VPC and then register the linked
        EC2-Classic instances with the load balancer in the VPC.

        Note that ``RegisterInstanceWithLoadBalancer`` completes when the
        request has been registered. Instance registration takes a little time
        to complete. To check the state of the registered instances, use
        DescribeLoadBalancers or DescribeInstanceHealth.

        After the instance is registered, it starts receiving traffic and
        requests from the load balancer. Any instance that is not in one of the
        Availability Zones registered for the load balancer is moved to the
        ``OutOfService`` state. If an Availability Zone is added to the load
        balancer later, any instances registered with the load balancer move to
        the ``InService`` state.

        To deregister instances from a load balancer, use
        DeregisterInstancesFromLoadBalancer.

        For more information, see `Register or De-Register EC2
        Instances <https://docs.aws.amazon.com/elasticloadbalancing/latest/classic/elb-deregister-register-instances.html>`__
        in the *Classic Load Balancers Guide*.

        :param load_balancer_name: The name of the load balancer.
        :param instances: The IDs of the instances.
        :returns: RegisterEndPointsOutput
        :raises AccessPointNotFoundException:
        :raises InvalidEndPointException:
        """
        raise NotImplementedError

    @handler("RemoveTags")
    def remove_tags(
        self, context: RequestContext, load_balancer_names: LoadBalancerNames, tags: TagKeyList
    ) -> RemoveTagsOutput:
        """Removes one or more tags from the specified load balancer.

        :param load_balancer_names: The name of the load balancer.
        :param tags: The list of tag keys to remove.
        :returns: RemoveTagsOutput
        :raises AccessPointNotFoundException:
        """
        raise NotImplementedError

    @handler("SetLoadBalancerListenerSSLCertificate")
    def set_load_balancer_listener_ssl_certificate(
        self,
        context: RequestContext,
        load_balancer_name: AccessPointName,
        load_balancer_port: AccessPointPort,
        ssl_certificate_id: SSLCertificateId,
    ) -> SetLoadBalancerListenerSSLCertificateOutput:
        """Sets the certificate that terminates the specified listener's SSL
        connections. The specified certificate replaces any prior certificate
        that was used on the same load balancer and port.

        For more information about updating your SSL certificate, see `Replace
        the SSL Certificate for Your Load
        Balancer <https://docs.aws.amazon.com/elasticloadbalancing/latest/classic/elb-update-ssl-cert.html>`__
        in the *Classic Load Balancers Guide*.

        :param load_balancer_name: The name of the load balancer.
        :param load_balancer_port: The port that uses the specified SSL certificate.
        :param ssl_certificate_id: The Amazon Resource Name (ARN) of the SSL certificate.
        :returns: SetLoadBalancerListenerSSLCertificateOutput
        :raises CertificateNotFoundException:
        :raises AccessPointNotFoundException:
        :raises ListenerNotFoundException:
        :raises InvalidConfigurationRequestException:
        :raises UnsupportedProtocolException:
        """
        raise NotImplementedError

    @handler("SetLoadBalancerPoliciesForBackendServer")
    def set_load_balancer_policies_for_backend_server(
        self,
        context: RequestContext,
        load_balancer_name: AccessPointName,
        instance_port: EndPointPort,
        policy_names: PolicyNames,
    ) -> SetLoadBalancerPoliciesForBackendServerOutput:
        """Replaces the set of policies associated with the specified port on which
        the EC2 instance is listening with a new set of policies. At this time,
        only the back-end server authentication policy type can be applied to
        the instance ports; this policy type is composed of multiple public key
        policies.

        Each time you use ``SetLoadBalancerPoliciesForBackendServer`` to enable
        the policies, use the ``PolicyNames`` parameter to list the policies
        that you want to enable.

        You can use DescribeLoadBalancers or DescribeLoadBalancerPolicies to
        verify that the policy is associated with the EC2 instance.

        For more information about enabling back-end instance authentication,
        see `Configure Back-end Instance
        Authentication <https://docs.aws.amazon.com/elasticloadbalancing/latest/classic/elb-create-https-ssl-load-balancer.html#configure_backendauth_clt>`__
        in the *Classic Load Balancers Guide*. For more information about Proxy
        Protocol, see `Configure Proxy Protocol
        Support <https://docs.aws.amazon.com/elasticloadbalancing/latest/classic/enable-proxy-protocol.html>`__
        in the *Classic Load Balancers Guide*.

        :param load_balancer_name: The name of the load balancer.
        :param instance_port: The port number associated with the EC2 instance.
        :param policy_names: The names of the policies.
        :returns: SetLoadBalancerPoliciesForBackendServerOutput
        :raises AccessPointNotFoundException:
        :raises PolicyNotFoundException:
        :raises InvalidConfigurationRequestException:
        """
        raise NotImplementedError

    @handler("SetLoadBalancerPoliciesOfListener")
    def set_load_balancer_policies_of_listener(
        self,
        context: RequestContext,
        load_balancer_name: AccessPointName,
        load_balancer_port: AccessPointPort,
        policy_names: PolicyNames,
    ) -> SetLoadBalancerPoliciesOfListenerOutput:
        """Replaces the current set of policies for the specified load balancer
        port with the specified set of policies.

        To enable back-end server authentication, use
        SetLoadBalancerPoliciesForBackendServer.

        For more information about setting policies, see `Update the SSL
        Negotiation
        Configuration <https://docs.aws.amazon.com/elasticloadbalancing/latest/classic/ssl-config-update.html>`__,
        `Duration-Based Session
        Stickiness <https://docs.aws.amazon.com/elasticloadbalancing/latest/classic/elb-sticky-sessions.html#enable-sticky-sessions-duration>`__,
        and `Application-Controlled Session
        Stickiness <https://docs.aws.amazon.com/elasticloadbalancing/latest/classic/elb-sticky-sessions.html#enable-sticky-sessions-application>`__
        in the *Classic Load Balancers Guide*.

        :param load_balancer_name: The name of the load balancer.
        :param load_balancer_port: The external port of the load balancer.
        :param policy_names: The names of the policies.
        :returns: SetLoadBalancerPoliciesOfListenerOutput
        :raises AccessPointNotFoundException:
        :raises PolicyNotFoundException:
        :raises ListenerNotFoundException:
        :raises InvalidConfigurationRequestException:
        """
        raise NotImplementedError



_O='policyName'
_N='ruleName'
_M='delete'
_L='thingName'
_K='LogicalResourceId'
_J='Properties'
_I='create'
_H='iot'
_G='certificateId'
_F='PolicyName'
_E='RuleName'
_D='parameters'
_C='ThingName'
_B='function'
_A=None
from typing import Dict
from localstack.services.cloudformation.deployment_utils import generate_default_name_without_stack,lambda_keys_to_lower,remove_none_values
from localstack.services.cloudformation.service_models import GenericBaseModel
from localstack.utils.aws import aws_stack
from localstack_ext.services.cloudformation.service_models import lambda_to_json
class IoTCertificate(GenericBaseModel):
	@staticmethod
	def cloudformation_type():return'AWS::IoT::Certificate'
	def fetch_state(A,stack_name,resources):return A.state.get(_G)
	def get_physical_resource_id(A,attribute=_A,**B):return A.props.get(_G)
	@classmethod
	def get_deploy_templates(C):
		def A(resource_id,resources,*F,**G):D=aws_stack.connect_to_service(_H);A=C(resources[resource_id]);E=A.props.get('CertificateSigningRequest');B=D.create_certificate_from_csr(certificateSigningRequest=E);A.state[_G]=B.get(_G);return B
		return{_I:{_B:A}}
class IoTThing(GenericBaseModel):
	@staticmethod
	def cloudformation_type():return'AWS::IoT::Thing'
	@staticmethod
	def add_defaults(resource,stack_name):
		A=resource;B=A[_J]
		if not B.get(_C):B[_C]=generate_default_name_without_stack(A[_K])
	def get_physical_resource_id(A,attribute=_A,**B):return A.props.get(_C)
	def fetch_state(A,stack_name,resources):B=A.props.get(_C);C=aws_stack.connect_to_service(_H);D=C.list_things(maxResults=500)['things'];E=[A for A in D if A.get(_L)==B];return(E or[_A])[0]
	@classmethod
	def get_deploy_templates(B):
		def A(params,**C):A=params;B={_L:A.get(_C),'thingTypeName':A.get('ThingTypeName'),'attributePayload':{'attributes':A.get('AttributePayload',{}).get('Attributes',{})}};return remove_none_values(B)
		return{_I:{_B:'create_thing',_D:A},_M:{_B:'delete_thing',_D:{_L:_C}}}
class IoTTopicRule(GenericBaseModel):
	@staticmethod
	def cloudformation_type():return'AWS::IoT::TopicRule'
	@staticmethod
	def add_defaults(resource,stack_name):
		A=resource;B=A[_J]
		if not B.get(_E):B[_E]=generate_default_name_without_stack(A[_K])
	def get_physical_resource_id(A,attribute=_A,**B):return A.props.get(_E)
	def fetch_state(A,stack_name,resources):B=A.props.get(_E);C=aws_stack.connect_to_service(_H);D=C.list_topic_rules()['rules'];E=[A for A in D if A.get(_N)==B];return(E or[_A])[0]
	@classmethod
	def get_deploy_templates(A):return{_I:{_B:'create_topic_rule',_D:{_N:_E,'topicRulePayload':lambda_keys_to_lower('TopicRulePayload'),'tags':lambda_keys_to_lower('Tags')},'types':{'ruleDisabled':bool}},_M:{_B:'delete_topic_rule',_D:{_N:_E}}}
class IoTPolicy(GenericBaseModel):
	@staticmethod
	def cloudformation_type():return'AWS::IoT::Policy'
	@staticmethod
	def add_defaults(resource,stack_name):
		A=resource;B=A[_J]
		if not B.get(_F):B[_F]=generate_default_name_without_stack(A[_K])
	def get_physical_resource_id(A,attribute=_A,**B):return A.props.get(_F)
	def fetch_state(A,stack_name,resources):B=A.props.get(_F);C=aws_stack.connect_to_service(_H);D=C.list_policies(pageSize=500)['policies'];E=[A for A in D if A.get(_O)==B];return(E or[_A])[0]
	@classmethod
	def get_deploy_templates(A):return{_I:{_B:'create_policy',_D:{_O:_F,'policyDocument':lambda_to_json('PolicyDocument')}},_M:{_B:'delete_policy',_D:{_O:_F}}}


import sys
from typing import Dict, List, Optional

if sys.version_info >= (3, 8):
    from typing import TypedDict
else:
    from typing_extensions import TypedDict

from localstack.aws.api import RequestContext, ServiceException, ServiceRequest, handler

Boolean = bool
Float = float
ImageIdOverride = str
ImageType = str
Integer = int
String = str
TagKey = str
TagValue = str


class ArrayJobDependency(str):
    N_TO_N = "N_TO_N"
    SEQUENTIAL = "SEQUENTIAL"


class AssignPublicIp(str):
    ENABLED = "ENABLED"
    DISABLED = "DISABLED"


class CEState(str):
    ENABLED = "ENABLED"
    DISABLED = "DISABLED"


class CEStatus(str):
    CREATING = "CREATING"
    UPDATING = "UPDATING"
    DELETING = "DELETING"
    DELETED = "DELETED"
    VALID = "VALID"
    INVALID = "INVALID"


class CEType(str):
    MANAGED = "MANAGED"
    UNMANAGED = "UNMANAGED"


class CRAllocationStrategy(str):
    BEST_FIT = "BEST_FIT"
    BEST_FIT_PROGRESSIVE = "BEST_FIT_PROGRESSIVE"
    SPOT_CAPACITY_OPTIMIZED = "SPOT_CAPACITY_OPTIMIZED"


class CRType(str):
    EC2 = "EC2"
    SPOT = "SPOT"
    FARGATE = "FARGATE"
    FARGATE_SPOT = "FARGATE_SPOT"


class CRUpdateAllocationStrategy(str):
    BEST_FIT_PROGRESSIVE = "BEST_FIT_PROGRESSIVE"
    SPOT_CAPACITY_OPTIMIZED = "SPOT_CAPACITY_OPTIMIZED"


class DeviceCgroupPermission(str):
    READ = "READ"
    WRITE = "WRITE"
    MKNOD = "MKNOD"


class EFSAuthorizationConfigIAM(str):
    ENABLED = "ENABLED"
    DISABLED = "DISABLED"


class EFSTransitEncryption(str):
    ENABLED = "ENABLED"
    DISABLED = "DISABLED"


class JQState(str):
    ENABLED = "ENABLED"
    DISABLED = "DISABLED"


class JQStatus(str):
    CREATING = "CREATING"
    UPDATING = "UPDATING"
    DELETING = "DELETING"
    DELETED = "DELETED"
    VALID = "VALID"
    INVALID = "INVALID"


class JobDefinitionType(str):
    container = "container"
    multinode = "multinode"


class JobStatus(str):
    SUBMITTED = "SUBMITTED"
    PENDING = "PENDING"
    RUNNABLE = "RUNNABLE"
    STARTING = "STARTING"
    RUNNING = "RUNNING"
    SUCCEEDED = "SUCCEEDED"
    FAILED = "FAILED"


class LogDriver(str):
    json_file = "json-file"
    syslog = "syslog"
    journald = "journald"
    gelf = "gelf"
    fluentd = "fluentd"
    awslogs = "awslogs"
    splunk = "splunk"


class PlatformCapability(str):
    EC2 = "EC2"
    FARGATE = "FARGATE"


class ResourceType(str):
    GPU = "GPU"
    VCPU = "VCPU"
    MEMORY = "MEMORY"


class RetryAction(str):
    RETRY = "RETRY"
    EXIT = "EXIT"


class ClientException(ServiceException):
    """These errors are usually caused by a client action, such as using an
    action or resource on behalf of a user that doesn't have permissions to
    use the action or resource, or specifying an identifier that's not
    valid.
    """

    message: Optional[String]


class ServerException(ServiceException):
    """These errors are usually caused by a server issue."""

    message: Optional[String]


ArrayJobStatusSummary = Dict[String, Integer]


class ArrayProperties(TypedDict, total=False):
    """An object representing an Batch array job."""

    size: Optional[Integer]


class ArrayPropertiesDetail(TypedDict, total=False):
    """An object representing the array properties of a job."""

    statusSummary: Optional[ArrayJobStatusSummary]
    size: Optional[Integer]
    index: Optional[Integer]


class ArrayPropertiesSummary(TypedDict, total=False):
    """An object representing the array properties of a job."""

    size: Optional[Integer]
    index: Optional[Integer]


class NetworkInterface(TypedDict, total=False):
    """An object representing the elastic network interface for a multi-node
    parallel job node.
    """

    attachmentId: Optional[String]
    ipv6Address: Optional[String]
    privateIpv4Address: Optional[String]


NetworkInterfaceList = List[NetworkInterface]


class AttemptContainerDetail(TypedDict, total=False):
    """An object representing the details of a container that's part of a job
    attempt.
    """

    containerInstanceArn: Optional[String]
    taskArn: Optional[String]
    exitCode: Optional[Integer]
    reason: Optional[String]
    logStreamName: Optional[String]
    networkInterfaces: Optional[NetworkInterfaceList]


Long = int


class AttemptDetail(TypedDict, total=False):
    """An object representing a job attempt."""

    container: Optional[AttemptContainerDetail]
    startedAt: Optional[Long]
    stoppedAt: Optional[Long]
    statusReason: Optional[String]


AttemptDetails = List[AttemptDetail]


class CancelJobRequest(ServiceRequest):
    """Contains the parameters for ``CancelJob``."""

    jobId: String
    reason: String


class CancelJobResponse(TypedDict, total=False):
    pass


JobExecutionTimeoutMinutes = int


class UpdatePolicy(TypedDict, total=False):
    """Specifies the infrastructure update policy for the compute environment.
    For more information about infrastructure updates, see `Infrastructure
    updates <https://docs.aws.amazon.com/batch/latest/userguide/infrastructure-updates.html>`__
    in the *Batch User Guide*.
    """

    terminateJobsOnUpdate: Optional[Boolean]
    jobExecutionTimeoutMinutes: Optional[JobExecutionTimeoutMinutes]


class Ec2Configuration(TypedDict, total=False):
    """Provides information used to select Amazon Machine Images (AMIs) for
    instances in the compute environment. If ``Ec2Configuration`` isn't
    specified, the default is ``ECS_AL2`` (`Amazon Linux
    2 <https://docs.aws.amazon.com/AmazonECS/latest/developerguide/ecs-optimized_AMI.html#al2ami>`__).

    This object isn't applicable to jobs that are running on Fargate
    resources.
    """

    imageType: ImageType
    imageIdOverride: Optional[ImageIdOverride]


Ec2ConfigurationList = List[Ec2Configuration]


class LaunchTemplateSpecification(TypedDict, total=False):
    """An object representing a launch template associated with a compute
    resource. You must specify either the launch template ID or launch
    template name in the request, but not both.

    If security groups are specified using both the ``securityGroupIds``
    parameter of ``CreateComputeEnvironment`` and the launch template, the
    values in the ``securityGroupIds`` parameter of
    ``CreateComputeEnvironment`` will be used.

    This object isn't applicable to jobs that are running on Fargate
    resources.
    """

    launchTemplateId: Optional[String]
    launchTemplateName: Optional[String]
    version: Optional[String]


TagsMap = Dict[String, String]
StringList = List[String]
ComputeResource = TypedDict(
    "ComputeResource",
    {
        "type": CRType,
        "allocationStrategy": Optional[CRAllocationStrategy],
        "minvCpus": Optional[Integer],
        "maxvCpus": Integer,
        "desiredvCpus": Optional[Integer],
        "instanceTypes": Optional[StringList],
        "imageId": Optional[String],
        "subnets": StringList,
        "securityGroupIds": Optional[StringList],
        "ec2KeyPair": Optional[String],
        "instanceRole": Optional[String],
        "tags": Optional[TagsMap],
        "placementGroup": Optional[String],
        "bidPercentage": Optional[Integer],
        "spotIamFleetRole": Optional[String],
        "launchTemplate": Optional[LaunchTemplateSpecification],
        "ec2Configuration": Optional[Ec2ConfigurationList],
    },
    total=False,
)
TagrisTagsMap = Dict[TagKey, TagValue]
ComputeEnvironmentDetail = TypedDict(
    "ComputeEnvironmentDetail",
    {
        "computeEnvironmentName": String,
        "computeEnvironmentArn": String,
        "unmanagedvCpus": Optional[Integer],
        "ecsClusterArn": Optional[String],
        "tags": Optional[TagrisTagsMap],
        "type": Optional[CEType],
        "state": Optional[CEState],
        "status": Optional[CEStatus],
        "statusReason": Optional[String],
        "computeResources": Optional[ComputeResource],
        "serviceRole": Optional[String],
        "updatePolicy": Optional[UpdatePolicy],
    },
    total=False,
)
ComputeEnvironmentDetailList = List[ComputeEnvironmentDetail]


class ComputeEnvironmentOrder(TypedDict, total=False):
    """The order in which compute environments are tried for job placement
    within a queue. Compute environments are tried in ascending order. For
    example, if two compute environments are associated with a job queue,
    the compute environment with a lower order integer value is tried for
    job placement first. Compute environments must be in the ``VALID`` state
    before you can associate them with a job queue. All of the compute
    environments must be either EC2 (``EC2`` or ``SPOT``) or Fargate
    (``FARGATE`` or ``FARGATE_SPOT``); EC2 and Fargate compute environments
    can't be mixed.

    All compute environments that are associated with a job queue must share
    the same architecture. Batch doesn't support mixing compute environment
    architecture types in a single job queue.
    """

    order: Integer
    computeEnvironment: String


ComputeEnvironmentOrders = List[ComputeEnvironmentOrder]
ComputeResourceUpdate = TypedDict(
    "ComputeResourceUpdate",
    {
        "minvCpus": Optional[Integer],
        "maxvCpus": Optional[Integer],
        "desiredvCpus": Optional[Integer],
        "subnets": Optional[StringList],
        "securityGroupIds": Optional[StringList],
        "allocationStrategy": Optional[CRUpdateAllocationStrategy],
        "instanceTypes": Optional[StringList],
        "ec2KeyPair": Optional[String],
        "instanceRole": Optional[String],
        "tags": Optional[TagsMap],
        "placementGroup": Optional[String],
        "bidPercentage": Optional[Integer],
        "launchTemplate": Optional[LaunchTemplateSpecification],
        "ec2Configuration": Optional[Ec2ConfigurationList],
        "updateToLatestImageVersion": Optional[Boolean],
        "type": Optional[CRType],
        "imageId": Optional[String],
    },
    total=False,
)


class FargatePlatformConfiguration(TypedDict, total=False):
    """The platform configuration for jobs that are running on Fargate
    resources. Jobs that run on EC2 resources must not specify this
    parameter.
    """

    platformVersion: Optional[String]


class NetworkConfiguration(TypedDict, total=False):
    """The network configuration for jobs that are running on Fargate
    resources. Jobs that are running on EC2 resources must not specify this
    parameter.
    """

    assignPublicIp: Optional[AssignPublicIp]


class Secret(TypedDict, total=False):
    """An object representing the secret to expose to your container. Secrets
    can be exposed to a container in the following ways:

    -  To inject sensitive data into your containers as environment
       variables, use the ``secrets`` container definition parameter.

    -  To reference sensitive information in the log configuration of a
       container, use the ``secretOptions`` container definition parameter.

    For more information, see `Specifying sensitive
    data <https://docs.aws.amazon.com/batch/latest/userguide/specifying-sensitive-data.html>`__
    in the *Batch User Guide*.
    """

    name: String
    valueFrom: String


SecretList = List[Secret]
LogConfigurationOptionsMap = Dict[String, String]


class LogConfiguration(TypedDict, total=False):
    """Log configuration options to send to a custom log driver for the
    container.
    """

    logDriver: LogDriver
    options: Optional[LogConfigurationOptionsMap]
    secretOptions: Optional[SecretList]


class Tmpfs(TypedDict, total=False):
    """The container path, mount options, and size of the tmpfs mount.

    This object isn't applicable to jobs that are running on Fargate
    resources.
    """

    containerPath: String
    size: Integer
    mountOptions: Optional[StringList]


TmpfsList = List[Tmpfs]
DeviceCgroupPermissions = List[DeviceCgroupPermission]


class Device(TypedDict, total=False):
    """An object representing a container instance host device.

    This object isn't applicable to jobs that are running on Fargate
    resources and shouldn't be provided.
    """

    hostPath: String
    containerPath: Optional[String]
    permissions: Optional[DeviceCgroupPermissions]


DevicesList = List[Device]


class LinuxParameters(TypedDict, total=False):
    """Linux-specific modifications that are applied to the container, such as
    details for device mappings.
    """

    devices: Optional[DevicesList]
    initProcessEnabled: Optional[Boolean]
    sharedMemorySize: Optional[Integer]
    tmpfs: Optional[TmpfsList]
    maxSwap: Optional[Integer]
    swappiness: Optional[Integer]


ResourceRequirement = TypedDict(
    "ResourceRequirement",
    {
        "value": String,
        "type": ResourceType,
    },
    total=False,
)
ResourceRequirements = List[ResourceRequirement]


class Ulimit(TypedDict, total=False):
    """The ``ulimit`` settings to pass to the container.

    This object isn't applicable to jobs that are running on Fargate
    resources.
    """

    hardLimit: Integer
    name: String
    softLimit: Integer


Ulimits = List[Ulimit]


class MountPoint(TypedDict, total=False):
    """Details on a Docker volume mount point that's used in a job's container
    properties. This parameter maps to ``Volumes`` in the `Create a
    container <https://docs.docker.com/engine/reference/api/docker_remote_api_v1.19/#create-a-container>`__
    section of the Docker Remote API and the ``--volume`` option to docker
    run.
    """

    containerPath: Optional[String]
    readOnly: Optional[Boolean]
    sourceVolume: Optional[String]


MountPoints = List[MountPoint]


class KeyValuePair(TypedDict, total=False):
    """A key-value pair object."""

    name: Optional[String]
    value: Optional[String]


EnvironmentVariables = List[KeyValuePair]


class EFSAuthorizationConfig(TypedDict, total=False):
    """The authorization configuration details for the Amazon EFS file system."""

    accessPointId: Optional[String]
    iam: Optional[EFSAuthorizationConfigIAM]


class EFSVolumeConfiguration(TypedDict, total=False):
    """This is used when you're using an Amazon Elastic File System file system
    for job storage. For more information, see `Amazon EFS
    Volumes <https://docs.aws.amazon.com/batch/latest/userguide/efs-volumes.html>`__
    in the *Batch User Guide*.
    """

    fileSystemId: String
    rootDirectory: Optional[String]
    transitEncryption: Optional[EFSTransitEncryption]
    transitEncryptionPort: Optional[Integer]
    authorizationConfig: Optional[EFSAuthorizationConfig]


class Host(TypedDict, total=False):
    """Determine whether your data volume persists on the host container
    instance and where it is stored. If this parameter is empty, then the
    Docker daemon assigns a host path for your data volume, but the data
    isn't guaranteed to persist after the containers associated with it stop
    running.
    """

    sourcePath: Optional[String]


class Volume(TypedDict, total=False):
    """A data volume used in a job's container properties."""

    host: Optional[Host]
    name: Optional[String]
    efsVolumeConfiguration: Optional[EFSVolumeConfiguration]


Volumes = List[Volume]


class ContainerDetail(TypedDict, total=False):
    """An object representing the details of a container that's part of a job."""

    image: Optional[String]
    vcpus: Optional[Integer]
    memory: Optional[Integer]
    command: Optional[StringList]
    jobRoleArn: Optional[String]
    executionRoleArn: Optional[String]
    volumes: Optional[Volumes]
    environment: Optional[EnvironmentVariables]
    mountPoints: Optional[MountPoints]
    readonlyRootFilesystem: Optional[Boolean]
    ulimits: Optional[Ulimits]
    privileged: Optional[Boolean]
    user: Optional[String]
    exitCode: Optional[Integer]
    reason: Optional[String]
    containerInstanceArn: Optional[String]
    taskArn: Optional[String]
    logStreamName: Optional[String]
    instanceType: Optional[String]
    networkInterfaces: Optional[NetworkInterfaceList]
    resourceRequirements: Optional[ResourceRequirements]
    linuxParameters: Optional[LinuxParameters]
    logConfiguration: Optional[LogConfiguration]
    secrets: Optional[SecretList]
    networkConfiguration: Optional[NetworkConfiguration]
    fargatePlatformConfiguration: Optional[FargatePlatformConfiguration]


class ContainerOverrides(TypedDict, total=False):
    """The overrides that should be sent to a container."""

    vcpus: Optional[Integer]
    memory: Optional[Integer]
    command: Optional[StringList]
    instanceType: Optional[String]
    environment: Optional[EnvironmentVariables]
    resourceRequirements: Optional[ResourceRequirements]


class ContainerProperties(TypedDict, total=False):
    """Container properties are used in job definitions to describe the
    container that's launched as part of a job.
    """

    image: Optional[String]
    vcpus: Optional[Integer]
    memory: Optional[Integer]
    command: Optional[StringList]
    jobRoleArn: Optional[String]
    executionRoleArn: Optional[String]
    volumes: Optional[Volumes]
    environment: Optional[EnvironmentVariables]
    mountPoints: Optional[MountPoints]
    readonlyRootFilesystem: Optional[Boolean]
    privileged: Optional[Boolean]
    ulimits: Optional[Ulimits]
    user: Optional[String]
    instanceType: Optional[String]
    resourceRequirements: Optional[ResourceRequirements]
    linuxParameters: Optional[LinuxParameters]
    logConfiguration: Optional[LogConfiguration]
    secrets: Optional[SecretList]
    networkConfiguration: Optional[NetworkConfiguration]
    fargatePlatformConfiguration: Optional[FargatePlatformConfiguration]


class ContainerSummary(TypedDict, total=False):
    """An object representing summary details of a container within a job."""

    exitCode: Optional[Integer]
    reason: Optional[String]


CreateComputeEnvironmentRequest = TypedDict(
    "CreateComputeEnvironmentRequest",
    {
        "computeEnvironmentName": String,
        "type": CEType,
        "state": Optional[CEState],
        "unmanagedvCpus": Optional[Integer],
        "computeResources": Optional[ComputeResource],
        "serviceRole": Optional[String],
        "tags": Optional[TagrisTagsMap],
    },
    total=False,
)


class CreateComputeEnvironmentResponse(TypedDict, total=False):
    computeEnvironmentName: Optional[String]
    computeEnvironmentArn: Optional[String]


class CreateJobQueueRequest(ServiceRequest):
    """Contains the parameters for ``CreateJobQueue``."""

    jobQueueName: String
    state: Optional[JQState]
    schedulingPolicyArn: Optional[String]
    priority: Integer
    computeEnvironmentOrder: ComputeEnvironmentOrders
    tags: Optional[TagrisTagsMap]


class CreateJobQueueResponse(TypedDict, total=False):
    jobQueueName: String
    jobQueueArn: String


class ShareAttributes(TypedDict, total=False):
    """Specifies the weights for the fair share identifiers for the fair share
    policy. Fair share identifiers that aren't included have a default
    weight of ``1.0``.
    """

    shareIdentifier: String
    weightFactor: Optional[Float]


ShareAttributesList = List[ShareAttributes]


class FairsharePolicy(TypedDict, total=False):
    """The fair share policy for a scheduling policy."""

    shareDecaySeconds: Optional[Integer]
    computeReservation: Optional[Integer]
    shareDistribution: Optional[ShareAttributesList]


class CreateSchedulingPolicyRequest(ServiceRequest):
    """Contains the parameters for ``CreateSchedulingPolicy``."""

    name: String
    fairsharePolicy: Optional[FairsharePolicy]
    tags: Optional[TagrisTagsMap]


class CreateSchedulingPolicyResponse(TypedDict, total=False):
    name: String
    arn: String


class DeleteComputeEnvironmentRequest(ServiceRequest):
    """Contains the parameters for ``DeleteComputeEnvironment``."""

    computeEnvironment: String


class DeleteComputeEnvironmentResponse(TypedDict, total=False):
    pass


class DeleteJobQueueRequest(ServiceRequest):
    """Contains the parameters for ``DeleteJobQueue``."""

    jobQueue: String


class DeleteJobQueueResponse(TypedDict, total=False):
    pass


class DeleteSchedulingPolicyRequest(ServiceRequest):
    """Contains the parameters for ``DeleteSchedulingPolicy``."""

    arn: String


class DeleteSchedulingPolicyResponse(TypedDict, total=False):
    pass


class DeregisterJobDefinitionRequest(ServiceRequest):
    jobDefinition: String


class DeregisterJobDefinitionResponse(TypedDict, total=False):
    pass


class DescribeComputeEnvironmentsRequest(ServiceRequest):
    """Contains the parameters for ``DescribeComputeEnvironments``."""

    computeEnvironments: Optional[StringList]
    maxResults: Optional[Integer]
    nextToken: Optional[String]


class DescribeComputeEnvironmentsResponse(TypedDict, total=False):
    computeEnvironments: Optional[ComputeEnvironmentDetailList]
    nextToken: Optional[String]


class DescribeJobDefinitionsRequest(ServiceRequest):
    """Contains the parameters for ``DescribeJobDefinitions``."""

    jobDefinitions: Optional[StringList]
    maxResults: Optional[Integer]
    jobDefinitionName: Optional[String]
    status: Optional[String]
    nextToken: Optional[String]


PlatformCapabilityList = List[PlatformCapability]


class NodeRangeProperty(TypedDict, total=False):
    """An object representing the properties of the node range for a multi-node
    parallel job.
    """

    targetNodes: String
    container: Optional[ContainerProperties]


NodeRangeProperties = List[NodeRangeProperty]


class NodeProperties(TypedDict, total=False):
    """An object representing the node properties of a multi-node parallel job."""

    numNodes: Integer
    mainNode: Integer
    nodeRangeProperties: NodeRangeProperties


class JobTimeout(TypedDict, total=False):
    """An object representing a job timeout configuration."""

    attemptDurationSeconds: Optional[Integer]


class EvaluateOnExit(TypedDict, total=False):
    """Specifies a set of conditions to be met, and an action to take
    (``RETRY`` or ``EXIT``) if all conditions are met.
    """

    onStatusReason: Optional[String]
    onReason: Optional[String]
    onExitCode: Optional[String]
    action: RetryAction


EvaluateOnExitList = List[EvaluateOnExit]


class RetryStrategy(TypedDict, total=False):
    """The retry strategy associated with a job. For more information, see
    `Automated job
    retries <https://docs.aws.amazon.com/batch/latest/userguide/job_retries.html>`__
    in the *Batch User Guide*.
    """

    attempts: Optional[Integer]
    evaluateOnExit: Optional[EvaluateOnExitList]


ParametersMap = Dict[String, String]
JobDefinition = TypedDict(
    "JobDefinition",
    {
        "jobDefinitionName": String,
        "jobDefinitionArn": String,
        "revision": Integer,
        "status": Optional[String],
        "type": String,
        "schedulingPriority": Optional[Integer],
        "parameters": Optional[ParametersMap],
        "retryStrategy": Optional[RetryStrategy],
        "containerProperties": Optional[ContainerProperties],
        "timeout": Optional[JobTimeout],
        "nodeProperties": Optional[NodeProperties],
        "tags": Optional[TagrisTagsMap],
        "propagateTags": Optional[Boolean],
        "platformCapabilities": Optional[PlatformCapabilityList],
    },
    total=False,
)
JobDefinitionList = List[JobDefinition]


class DescribeJobDefinitionsResponse(TypedDict, total=False):
    jobDefinitions: Optional[JobDefinitionList]
    nextToken: Optional[String]


class DescribeJobQueuesRequest(ServiceRequest):
    """Contains the parameters for ``DescribeJobQueues``."""

    jobQueues: Optional[StringList]
    maxResults: Optional[Integer]
    nextToken: Optional[String]


class JobQueueDetail(TypedDict, total=False):
    """An object representing the details of an Batch job queue."""

    jobQueueName: String
    jobQueueArn: String
    state: JQState
    schedulingPolicyArn: Optional[String]
    status: Optional[JQStatus]
    statusReason: Optional[String]
    priority: Integer
    computeEnvironmentOrder: ComputeEnvironmentOrders
    tags: Optional[TagrisTagsMap]


JobQueueDetailList = List[JobQueueDetail]


class DescribeJobQueuesResponse(TypedDict, total=False):
    jobQueues: Optional[JobQueueDetailList]
    nextToken: Optional[String]


class DescribeJobsRequest(ServiceRequest):
    """Contains the parameters for ``DescribeJobs``."""

    jobs: StringList


class NodeDetails(TypedDict, total=False):
    """An object representing the details of a multi-node parallel job node."""

    nodeIndex: Optional[Integer]
    isMainNode: Optional[Boolean]


JobDependency = TypedDict(
    "JobDependency",
    {
        "jobId": Optional[String],
        "type": Optional[ArrayJobDependency],
    },
    total=False,
)
JobDependencyList = List[JobDependency]


class JobDetail(TypedDict, total=False):
    """An object representing an Batch job."""

    jobArn: Optional[String]
    jobName: String
    jobId: String
    jobQueue: String
    status: JobStatus
    shareIdentifier: Optional[String]
    schedulingPriority: Optional[Integer]
    attempts: Optional[AttemptDetails]
    statusReason: Optional[String]
    createdAt: Optional[Long]
    retryStrategy: Optional[RetryStrategy]
    startedAt: Long
    stoppedAt: Optional[Long]
    dependsOn: Optional[JobDependencyList]
    jobDefinition: String
    parameters: Optional[ParametersMap]
    container: Optional[ContainerDetail]
    nodeDetails: Optional[NodeDetails]
    nodeProperties: Optional[NodeProperties]
    arrayProperties: Optional[ArrayPropertiesDetail]
    timeout: Optional[JobTimeout]
    tags: Optional[TagrisTagsMap]
    propagateTags: Optional[Boolean]
    platformCapabilities: Optional[PlatformCapabilityList]


JobDetailList = List[JobDetail]


class DescribeJobsResponse(TypedDict, total=False):
    jobs: Optional[JobDetailList]


class DescribeSchedulingPoliciesRequest(ServiceRequest):
    """Contains the parameters for ``DescribeSchedulingPolicies``."""

    arns: StringList


class SchedulingPolicyDetail(TypedDict, total=False):
    """An object that represents a scheduling policy."""

    name: String
    arn: String
    fairsharePolicy: Optional[FairsharePolicy]
    tags: Optional[TagrisTagsMap]


SchedulingPolicyDetailList = List[SchedulingPolicyDetail]


class DescribeSchedulingPoliciesResponse(TypedDict, total=False):
    schedulingPolicies: Optional[SchedulingPolicyDetailList]


class NodePropertiesSummary(TypedDict, total=False):
    """An object representing the properties of a node that's associated with a
    multi-node parallel job.
    """

    isMainNode: Optional[Boolean]
    numNodes: Optional[Integer]
    nodeIndex: Optional[Integer]


class JobSummary(TypedDict, total=False):
    """An object representing summary details of a job."""

    jobArn: Optional[String]
    jobId: String
    jobName: String
    createdAt: Optional[Long]
    status: Optional[JobStatus]
    statusReason: Optional[String]
    startedAt: Optional[Long]
    stoppedAt: Optional[Long]
    container: Optional[ContainerSummary]
    arrayProperties: Optional[ArrayPropertiesSummary]
    nodeProperties: Optional[NodePropertiesSummary]
    jobDefinition: Optional[String]


JobSummaryList = List[JobSummary]


class KeyValuesPair(TypedDict, total=False):
    """A filter name and value pair that's used to return a more specific list
    of results from a ``ListJobs`` API operation.
    """

    name: Optional[String]
    values: Optional[StringList]


ListJobsFilterList = List[KeyValuesPair]


class ListJobsRequest(ServiceRequest):
    """Contains the parameters for ``ListJobs``."""

    jobQueue: Optional[String]
    arrayJobId: Optional[String]
    multiNodeJobId: Optional[String]
    jobStatus: Optional[JobStatus]
    maxResults: Optional[Integer]
    nextToken: Optional[String]
    filters: Optional[ListJobsFilterList]


class ListJobsResponse(TypedDict, total=False):
    jobSummaryList: JobSummaryList
    nextToken: Optional[String]


class ListSchedulingPoliciesRequest(ServiceRequest):
    """Contains the parameters for ``ListSchedulingPolicies``."""

    maxResults: Optional[Integer]
    nextToken: Optional[String]


class SchedulingPolicyListingDetail(TypedDict, total=False):
    """An object that contains the details of a scheduling policy that's
    returned in a ``ListSchedulingPolicy`` action.
    """

    arn: String


SchedulingPolicyListingDetailList = List[SchedulingPolicyListingDetail]


class ListSchedulingPoliciesResponse(TypedDict, total=False):
    schedulingPolicies: Optional[SchedulingPolicyListingDetailList]
    nextToken: Optional[String]


class ListTagsForResourceRequest(ServiceRequest):
    """Contains the parameters for ``ListTagsForResource``."""

    resourceArn: String


class ListTagsForResourceResponse(TypedDict, total=False):
    tags: Optional[TagrisTagsMap]


class NodePropertyOverride(TypedDict, total=False):
    """Object representing any node overrides to a job definition that's used
    in a SubmitJob API operation.
    """

    targetNodes: String
    containerOverrides: Optional[ContainerOverrides]


NodePropertyOverrides = List[NodePropertyOverride]


class NodeOverrides(TypedDict, total=False):
    """Object representing any node overrides to a job definition that's used
    in a SubmitJob API operation.

    This isn't applicable to jobs that are running on Fargate resources and
    shouldn't be provided; use ``containerOverrides`` instead.
    """

    numNodes: Optional[Integer]
    nodePropertyOverrides: Optional[NodePropertyOverrides]


RegisterJobDefinitionRequest = TypedDict(
    "RegisterJobDefinitionRequest",
    {
        "jobDefinitionName": String,
        "type": JobDefinitionType,
        "parameters": Optional[ParametersMap],
        "schedulingPriority": Optional[Integer],
        "containerProperties": Optional[ContainerProperties],
        "nodeProperties": Optional[NodeProperties],
        "retryStrategy": Optional[RetryStrategy],
        "propagateTags": Optional[Boolean],
        "timeout": Optional[JobTimeout],
        "tags": Optional[TagrisTagsMap],
        "platformCapabilities": Optional[PlatformCapabilityList],
    },
    total=False,
)


class RegisterJobDefinitionResponse(TypedDict, total=False):
    jobDefinitionName: String
    jobDefinitionArn: String
    revision: Integer


class SubmitJobRequest(ServiceRequest):
    """Contains the parameters for ``SubmitJob``."""

    jobName: String
    jobQueue: String
    shareIdentifier: Optional[String]
    schedulingPriorityOverride: Optional[Integer]
    arrayProperties: Optional[ArrayProperties]
    dependsOn: Optional[JobDependencyList]
    jobDefinition: String
    parameters: Optional[ParametersMap]
    containerOverrides: Optional[ContainerOverrides]
    nodeOverrides: Optional[NodeOverrides]
    retryStrategy: Optional[RetryStrategy]
    propagateTags: Optional[Boolean]
    timeout: Optional[JobTimeout]
    tags: Optional[TagrisTagsMap]


class SubmitJobResponse(TypedDict, total=False):
    jobArn: Optional[String]
    jobName: String
    jobId: String


TagKeysList = List[TagKey]


class TagResourceRequest(ServiceRequest):
    """Contains the parameters for ``TagResource``."""

    resourceArn: String
    tags: TagrisTagsMap


class TagResourceResponse(TypedDict, total=False):
    pass


class TerminateJobRequest(ServiceRequest):
    """Contains the parameters for ``TerminateJob``."""

    jobId: String
    reason: String


class TerminateJobResponse(TypedDict, total=False):
    pass


class UntagResourceRequest(ServiceRequest):
    """Contains the parameters for ``UntagResource``."""

    resourceArn: String
    tagKeys: TagKeysList


class UntagResourceResponse(TypedDict, total=False):
    pass


class UpdateComputeEnvironmentRequest(ServiceRequest):
    """Contains the parameters for ``UpdateComputeEnvironment``."""

    computeEnvironment: String
    state: Optional[CEState]
    unmanagedvCpus: Optional[Integer]
    computeResources: Optional[ComputeResourceUpdate]
    serviceRole: Optional[String]
    updatePolicy: Optional[UpdatePolicy]


class UpdateComputeEnvironmentResponse(TypedDict, total=False):
    computeEnvironmentName: Optional[String]
    computeEnvironmentArn: Optional[String]


class UpdateJobQueueRequest(ServiceRequest):
    """Contains the parameters for ``UpdateJobQueue``."""

    jobQueue: String
    state: Optional[JQState]
    schedulingPolicyArn: Optional[String]
    priority: Optional[Integer]
    computeEnvironmentOrder: Optional[ComputeEnvironmentOrders]


class UpdateJobQueueResponse(TypedDict, total=False):
    jobQueueName: Optional[String]
    jobQueueArn: Optional[String]


class UpdateSchedulingPolicyRequest(ServiceRequest):
    """Contains the parameters for ``UpdateSchedulingPolicy``."""

    arn: String
    fairsharePolicy: Optional[FairsharePolicy]


class UpdateSchedulingPolicyResponse(TypedDict, total=False):
    pass


class BatchApi:

    service = "batch"
    version = "2016-08-10"

    @handler("CancelJob")
    def cancel_job(
        self, context: RequestContext, job_id: String, reason: String
    ) -> CancelJobResponse:
        """Cancels a job in an Batch job queue. Jobs that are in the ``SUBMITTED``,
        ``PENDING``, or ``RUNNABLE`` state are canceled. Jobs that have
        progressed to ``STARTING`` or ``RUNNING`` aren't canceled, but the API
        operation still succeeds, even if no job is canceled. These jobs must be
        terminated with the TerminateJob operation.

        :param job_id: The Batch job ID of the job to cancel.
        :param reason: A message to attach to the job that explains the reason for canceling
        it.
        :returns: CancelJobResponse
        :raises ClientException:
        :raises ServerException:
        """
        raise NotImplementedError

    @handler("CreateComputeEnvironment", expand=False)
    def create_compute_environment(
        self, context: RequestContext, request: CreateComputeEnvironmentRequest
    ) -> CreateComputeEnvironmentResponse:
        """Creates an Batch compute environment. You can create ``MANAGED`` or
        ``UNMANAGED`` compute environments. ``MANAGED`` compute environments can
        use Amazon EC2 or Fargate resources. ``UNMANAGED`` compute environments
        can only use EC2 resources.

        In a managed compute environment, Batch manages the capacity and
        instance types of the compute resources within the environment. This is
        based on the compute resource specification that you define or the
        `launch
        template <https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-launch-templates.html>`__
        that you specify when you create the compute environment. Either, you
        can choose to use EC2 On-Demand Instances and EC2 Spot Instances. Or,
        you can use Fargate and Fargate Spot capacity in your managed compute
        environment. You can optionally set a maximum price so that Spot
        Instances only launch when the Spot Instance price is less than a
        specified percentage of the On-Demand price.

        Multi-node parallel jobs aren't supported on Spot Instances.

        In an unmanaged compute environment, you can manage your own EC2 compute
        resources and have a lot of flexibility with how you configure your
        compute resources. For example, you can use custom AMIs. However, you
        must verify that each of your AMIs meet the Amazon ECS container
        instance AMI specification. For more information, see `container
        instance
        AMIs <https://docs.aws.amazon.com/AmazonECS/latest/developerguide/container_instance_AMIs.html>`__
        in the *Amazon Elastic Container Service Developer Guide*. After you
        created your unmanaged compute environment, you can use the
        DescribeComputeEnvironments operation to find the Amazon ECS cluster
        that's associated with it. Then, launch your container instances into
        that Amazon ECS cluster. For more information, see `Launching an Amazon
        ECS container
        instance <https://docs.aws.amazon.com/AmazonECS/latest/developerguide/launch_container_instance.html>`__
        in the *Amazon Elastic Container Service Developer Guide*.

        Batch doesn't automatically upgrade the AMIs in a compute environment
        after it's created. For example, it also doesn't update the AMIs in your
        compute environment when a newer version of the Amazon ECS optimized AMI
        is available. You're responsible for the management of the guest
        operating system. This includes any updates and security patches. You're
        also responsible for any additional application software or utilities
        that you install on the compute resources. There are two ways to use a
        new AMI for your Batch jobs. The original method is to complete these
        steps:

        #. Create a new compute environment with the new AMI.

        #. Add the compute environment to an existing job queue.

        #. Remove the earlier compute environment from your job queue.

        #. Delete the earlier compute environment.

        In April 2022, Batch added enhanced support for updating compute
        environments. For more information, see `Updating compute
        environments <https://docs.aws.amazon.com/batch/latest/userguide/updating-compute-environments.html>`__.
        To use the enhanced updating of compute environments to update AMIs,
        follow these rules:

        -  Either do not set the service role (``serviceRole``) parameter or set
           it to the **AWSBatchServiceRole** service-linked role.

        -  Set the allocation strategy (``allocationStrategy``) parameter to
           ``BEST_FIT_PROGRESSIVE`` or ``SPOT_CAPACITY_OPTIMIZED``.

        -  Set the update to latest image version
           (``updateToLatestImageVersion``) parameter to ``true``.

        -  Do not specify an AMI ID in ``imageId``, ``imageIdOverride`` (in
           ```ec2Configuration`` <https://docs.aws.amazon.com/batch/latest/APIReference/API_Ec2Configuration.html>`__
           ), or in the launch template (``launchTemplate``). In that case Batch
           will select the latest Amazon ECS optimized AMI supported by Batch at
           the time the infrastructure update is initiated. Alternatively you
           can specify the AMI ID in the ``imageId`` or ``imageIdOverride``
           parameters, or the launch template identified by the
           ``LaunchTemplate`` properties. Changing any of these properties will
           trigger an infrastructure update. If the AMI ID is specified in the
           launch template, it can not be replaced by specifying an AMI ID in
           either the ``imageId`` or ``imageIdOverride`` parameters. It can only
           be replaced by specifying a different launch template, or if the
           launch template version is set to ``$Default`` or ``$Latest``, by
           setting either a new default version for the launch template (if
           ``$Default``)or by adding a new version to the launch template (if
           ``$Latest``).

        If these rules are followed, any update that triggers an infrastructure
        update will cause the AMI ID to be re-selected. If the ``version``
        setting in the launch template (``launchTemplate``) is set to
        ``$Latest`` or ``$Default``, the latest or default version of the launch
        template will be evaluated up at the time of the infrastructure update,
        even if the ``launchTemplate`` was not updated.

        :param compute_environment_name: The name for your compute environment.
        :param type: The type of the compute environment: ``MANAGED`` or ``UNMANAGED``.
        :param state: The state of the compute environment.
        :param unmanagedv_cpus: The maximum number of vCPUs for an unmanaged compute environment.
        :param compute_resources: Details about the compute resources managed by the compute environment.
        :param service_role: The full Amazon Resource Name (ARN) of the IAM role that allows Batch to
        make calls to other Amazon Web Services services on your behalf.
        :param tags: The tags that you apply to the compute environment to help you
        categorize and organize your resources.
        :returns: CreateComputeEnvironmentResponse
        :raises ClientException:
        :raises ServerException:
        """
        raise NotImplementedError

    @handler("CreateJobQueue")
    def create_job_queue(
        self,
        context: RequestContext,
        job_queue_name: String,
        priority: Integer,
        compute_environment_order: ComputeEnvironmentOrders,
        state: JQState = None,
        scheduling_policy_arn: String = None,
        tags: TagrisTagsMap = None,
    ) -> CreateJobQueueResponse:
        """Creates an Batch job queue. When you create a job queue, you associate
        one or more compute environments to the queue and assign an order of
        preference for the compute environments.

        You also set a priority to the job queue that determines the order that
        the Batch scheduler places jobs onto its associated compute
        environments. For example, if a compute environment is associated with
        more than one job queue, the job queue with a higher priority is given
        preference for scheduling jobs to that compute environment.

        :param job_queue_name: The name of the job queue.
        :param priority: The priority of the job queue.
        :param compute_environment_order: The set of compute environments mapped to a job queue and their order
        relative to each other.
        :param state: The state of the job queue.
        :param scheduling_policy_arn: The Amazon Resource Name (ARN) of the fair share scheduling policy.
        :param tags: The tags that you apply to the job queue to help you categorize and
        organize your resources.
        :returns: CreateJobQueueResponse
        :raises ClientException:
        :raises ServerException:
        """
        raise NotImplementedError

    @handler("CreateSchedulingPolicy")
    def create_scheduling_policy(
        self,
        context: RequestContext,
        name: String,
        fairshare_policy: FairsharePolicy = None,
        tags: TagrisTagsMap = None,
    ) -> CreateSchedulingPolicyResponse:
        """Creates an Batch scheduling policy.

        :param name: The name of the scheduling policy.
        :param fairshare_policy: The fair share policy of the scheduling policy.
        :param tags: The tags that you apply to the scheduling policy to help you categorize
        and organize your resources.
        :returns: CreateSchedulingPolicyResponse
        :raises ClientException:
        :raises ServerException:
        """
        raise NotImplementedError

    @handler("DeleteComputeEnvironment")
    def delete_compute_environment(
        self, context: RequestContext, compute_environment: String
    ) -> DeleteComputeEnvironmentResponse:
        """Deletes an Batch compute environment.

        Before you can delete a compute environment, you must set its state to
        ``DISABLED`` with the UpdateComputeEnvironment API operation and
        disassociate it from any job queues with the UpdateJobQueue API
        operation. Compute environments that use Fargate resources must
        terminate all active jobs on that compute environment before deleting
        the compute environment. If this isn't done, the compute environment
        enters an invalid state.

        :param compute_environment: The name or Amazon Resource Name (ARN) of the compute environment to
        delete.
        :returns: DeleteComputeEnvironmentResponse
        :raises ClientException:
        :raises ServerException:
        """
        raise NotImplementedError

    @handler("DeleteJobQueue")
    def delete_job_queue(
        self, context: RequestContext, job_queue: String
    ) -> DeleteJobQueueResponse:
        """Deletes the specified job queue. You must first disable submissions for
        a queue with the UpdateJobQueue operation. All jobs in the queue are
        eventually terminated when you delete a job queue. The jobs are
        terminated at a rate of about 16 jobs each second.

        It's not necessary to disassociate compute environments from a queue
        before submitting a ``DeleteJobQueue`` request.

        :param job_queue: The short name or full Amazon Resource Name (ARN) of the queue to
        delete.
        :returns: DeleteJobQueueResponse
        :raises ClientException:
        :raises ServerException:
        """
        raise NotImplementedError

    @handler("DeleteSchedulingPolicy")
    def delete_scheduling_policy(
        self, context: RequestContext, arn: String
    ) -> DeleteSchedulingPolicyResponse:
        """Deletes the specified scheduling policy.

        You can't delete a scheduling policy that's used in any job queues.

        :param arn: The Amazon Resource Name (ARN) of the scheduling policy to delete.
        :returns: DeleteSchedulingPolicyResponse
        :raises ClientException:
        :raises ServerException:
        """
        raise NotImplementedError

    @handler("DeregisterJobDefinition")
    def deregister_job_definition(
        self, context: RequestContext, job_definition: String
    ) -> DeregisterJobDefinitionResponse:
        """Deregisters an Batch job definition. Job definitions are permanently
        deleted after 180 days.

        :param job_definition: The name and revision (``name:revision``) or full Amazon Resource Name
        (ARN) of the job definition to deregister.
        :returns: DeregisterJobDefinitionResponse
        :raises ClientException:
        :raises ServerException:
        """
        raise NotImplementedError

    @handler("DescribeComputeEnvironments")
    def describe_compute_environments(
        self,
        context: RequestContext,
        compute_environments: StringList = None,
        max_results: Integer = None,
        next_token: String = None,
    ) -> DescribeComputeEnvironmentsResponse:
        """Describes one or more of your compute environments.

        If you're using an unmanaged compute environment, you can use the
        ``DescribeComputeEnvironment`` operation to determine the
        ``ecsClusterArn`` that you launch your Amazon ECS container instances
        into.

        :param compute_environments: A list of up to 100 compute environment names or full Amazon Resource
        Name (ARN) entries.
        :param max_results: The maximum number of cluster results returned by
        ``DescribeComputeEnvironments`` in paginated output.
        :param next_token: The ``nextToken`` value returned from a previous paginated
        ``DescribeComputeEnvironments`` request where ``maxResults`` was used
        and the results exceeded the value of that parameter.
        :returns: DescribeComputeEnvironmentsResponse
        :raises ClientException:
        :raises ServerException:
        """
        raise NotImplementedError

    @handler("DescribeJobDefinitions")
    def describe_job_definitions(
        self,
        context: RequestContext,
        job_definitions: StringList = None,
        max_results: Integer = None,
        job_definition_name: String = None,
        status: String = None,
        next_token: String = None,
    ) -> DescribeJobDefinitionsResponse:
        """Describes a list of job definitions. You can specify a ``status`` (such
        as ``ACTIVE``) to only return job definitions that match that status.

        :param job_definitions: A list of up to 100 job definitions.
        :param max_results: The maximum number of results returned by ``DescribeJobDefinitions`` in
        paginated output.
        :param job_definition_name: The name of the job definition to describe.
        :param status: The status used to filter job definitions.
        :param next_token: The ``nextToken`` value returned from a previous paginated
        ``DescribeJobDefinitions`` request where ``maxResults`` was used and the
        results exceeded the value of that parameter.
        :returns: DescribeJobDefinitionsResponse
        :raises ClientException:
        :raises ServerException:
        """
        raise NotImplementedError

    @handler("DescribeJobQueues")
    def describe_job_queues(
        self,
        context: RequestContext,
        job_queues: StringList = None,
        max_results: Integer = None,
        next_token: String = None,
    ) -> DescribeJobQueuesResponse:
        """Describes one or more of your job queues.

        :param job_queues: A list of up to 100 queue names or full queue Amazon Resource Name (ARN)
        entries.
        :param max_results: The maximum number of results returned by ``DescribeJobQueues`` in
        paginated output.
        :param next_token: The ``nextToken`` value returned from a previous paginated
        ``DescribeJobQueues`` request where ``maxResults`` was used and the
        results exceeded the value of that parameter.
        :returns: DescribeJobQueuesResponse
        :raises ClientException:
        :raises ServerException:
        """
        raise NotImplementedError

    @handler("DescribeJobs")
    def describe_jobs(self, context: RequestContext, jobs: StringList) -> DescribeJobsResponse:
        """Describes a list of Batch jobs.

        :param jobs: A list of up to 100 job IDs.
        :returns: DescribeJobsResponse
        :raises ClientException:
        :raises ServerException:
        """
        raise NotImplementedError

    @handler("DescribeSchedulingPolicies")
    def describe_scheduling_policies(
        self, context: RequestContext, arns: StringList
    ) -> DescribeSchedulingPoliciesResponse:
        """Describes one or more of your scheduling policies.

        :param arns: A list of up to 100 scheduling policy Amazon Resource Name (ARN)
        entries.
        :returns: DescribeSchedulingPoliciesResponse
        :raises ClientException:
        :raises ServerException:
        """
        raise NotImplementedError

    @handler("ListJobs")
    def list_jobs(
        self,
        context: RequestContext,
        job_queue: String = None,
        array_job_id: String = None,
        multi_node_job_id: String = None,
        job_status: JobStatus = None,
        max_results: Integer = None,
        next_token: String = None,
        filters: ListJobsFilterList = None,
    ) -> ListJobsResponse:
        """Returns a list of Batch jobs.

        You must specify only one of the following items:

        -  A job queue ID to return a list of jobs in that job queue

        -  A multi-node parallel job ID to return a list of nodes for that job

        -  An array job ID to return a list of the children for that job

        You can filter the results by job status with the ``jobStatus``
        parameter. If you don't specify a status, only ``RUNNING`` jobs are
        returned.

        :param job_queue: The name or full Amazon Resource Name (ARN) of the job queue used to
        list jobs.
        :param array_job_id: The job ID for an array job.
        :param multi_node_job_id: The job ID for a multi-node parallel job.
        :param job_status: The job status used to filter jobs in the specified queue.
        :param max_results: The maximum number of results returned by ``ListJobs`` in paginated
        output.
        :param next_token: The ``nextToken`` value returned from a previous paginated ``ListJobs``
        request where ``maxResults`` was used and the results exceeded the value
        of that parameter.
        :param filters: The filter to apply to the query.
        :returns: ListJobsResponse
        :raises ClientException:
        :raises ServerException:
        """
        raise NotImplementedError

    @handler("ListSchedulingPolicies")
    def list_scheduling_policies(
        self, context: RequestContext, max_results: Integer = None, next_token: String = None
    ) -> ListSchedulingPoliciesResponse:
        """Returns a list of Batch scheduling policies.

        :param max_results: The maximum number of results that's returned by
        ``ListSchedulingPolicies`` in paginated output.
        :param next_token: The ``nextToken`` value that's returned from a previous paginated
        ``ListSchedulingPolicies`` request where ``maxResults`` was used and the
        results exceeded the value of that parameter.
        :returns: ListSchedulingPoliciesResponse
        :raises ClientException:
        :raises ServerException:
        """
        raise NotImplementedError

    @handler("ListTagsForResource")
    def list_tags_for_resource(
        self, context: RequestContext, resource_arn: String
    ) -> ListTagsForResourceResponse:
        """Lists the tags for an Batch resource. Batch resources that support tags
        are compute environments, jobs, job definitions, job queues, and
        scheduling policies. ARNs for child jobs of array and multi-node
        parallel (MNP) jobs are not supported.

        :param resource_arn: The Amazon Resource Name (ARN) that identifies the resource that tags
        are listed for.
        :returns: ListTagsForResourceResponse
        :raises ClientException:
        :raises ServerException:
        """
        raise NotImplementedError

    @handler("RegisterJobDefinition", expand=False)
    def register_job_definition(
        self, context: RequestContext, request: RegisterJobDefinitionRequest
    ) -> RegisterJobDefinitionResponse:
        """Registers an Batch job definition.

        :param job_definition_name: The name of the job definition to register.
        :param type: The type of job definition.
        :param parameters: Default parameter substitution placeholders to set in the job
        definition.
        :param scheduling_priority: The scheduling priority for jobs that are submitted with this job
        definition.
        :param container_properties: An object with various properties specific to single-node
        container-based jobs.
        :param node_properties: An object with various properties specific to multi-node parallel jobs.
        :param retry_strategy: The retry strategy to use for failed jobs that are submitted with this
        job definition.
        :param propagate_tags: Specifies whether to propagate the tags from the job or job definition
        to the corresponding Amazon ECS task.
        :param timeout: The timeout configuration for jobs that are submitted with this job
        definition, after which Batch terminates your jobs if they have not
        finished.
        :param tags: The tags that you apply to the job definition to help you categorize and
        organize your resources.
        :param platform_capabilities: The platform capabilities required by the job definition.
        :returns: RegisterJobDefinitionResponse
        :raises ClientException:
        :raises ServerException:
        """
        raise NotImplementedError

    @handler("SubmitJob")
    def submit_job(
        self,
        context: RequestContext,
        job_name: String,
        job_queue: String,
        job_definition: String,
        share_identifier: String = None,
        scheduling_priority_override: Integer = None,
        array_properties: ArrayProperties = None,
        depends_on: JobDependencyList = None,
        parameters: ParametersMap = None,
        container_overrides: ContainerOverrides = None,
        node_overrides: NodeOverrides = None,
        retry_strategy: RetryStrategy = None,
        propagate_tags: Boolean = None,
        timeout: JobTimeout = None,
        tags: TagrisTagsMap = None,
    ) -> SubmitJobResponse:
        """Submits an Batch job from a job definition. Parameters that are
        specified during SubmitJob override parameters defined in the job
        definition. vCPU and memory requirements that are specified in the
        ``resourceRequirements`` objects in the job definition are the
        exception. They can't be overridden this way using the ``memory`` and
        ``vcpus`` parameters. Rather, you must specify updates to job definition
        parameters in a ``resourceRequirements`` object that's included in the
        ``containerOverrides`` parameter.

        Job queues with a scheduling policy are limited to 500 active fair share
        identifiers at a time.

        Jobs that run on Fargate resources can't be guaranteed to run for more
        than 14 days. This is because, after 14 days, Fargate resources might
        become unavailable and job might be terminated.

        :param job_name: The name of the job.
        :param job_queue: The job queue where the job is submitted.
        :param job_definition: The job definition used by this job.
        :param share_identifier: The share identifier for the job.
        :param scheduling_priority_override: The scheduling priority for the job.
        :param array_properties: The array properties for the submitted job, such as the size of the
        array.
        :param depends_on: A list of dependencies for the job.
        :param parameters: Additional parameters passed to the job that replace parameter
        substitution placeholders that are set in the job definition.
        :param container_overrides: A list of container overrides in the JSON format that specify the name
        of a container in the specified job definition and the overrides it
        receives.
        :param node_overrides: A list of node overrides in JSON format that specify the node range to
        target and the container overrides for that node range.
        :param retry_strategy: The retry strategy to use for failed jobs from this SubmitJob operation.
        :param propagate_tags: Specifies whether to propagate the tags from the job or job definition
        to the corresponding Amazon ECS task.
        :param timeout: The timeout configuration for this SubmitJob operation.
        :param tags: The tags that you apply to the job request to help you categorize and
        organize your resources.
        :returns: SubmitJobResponse
        :raises ClientException:
        :raises ServerException:
        """
        raise NotImplementedError

    @handler("TagResource")
    def tag_resource(
        self, context: RequestContext, resource_arn: String, tags: TagrisTagsMap
    ) -> TagResourceResponse:
        """Associates the specified tags to a resource with the specified
        ``resourceArn``. If existing tags on a resource aren't specified in the
        request parameters, they aren't changed. When a resource is deleted, the
        tags that are associated with that resource are deleted as well. Batch
        resources that support tags are compute environments, jobs, job
        definitions, job queues, and scheduling policies. ARNs for child jobs of
        array and multi-node parallel (MNP) jobs are not supported.

        :param resource_arn: The Amazon Resource Name (ARN) of the resource that tags are added to.
        :param tags: The tags that you apply to the resource to help you categorize and
        organize your resources.
        :returns: TagResourceResponse
        :raises ClientException:
        :raises ServerException:
        """
        raise NotImplementedError

    @handler("TerminateJob")
    def terminate_job(
        self, context: RequestContext, job_id: String, reason: String
    ) -> TerminateJobResponse:
        """Terminates a job in a job queue. Jobs that are in the ``STARTING`` or
        ``RUNNING`` state are terminated, which causes them to transition to
        ``FAILED``. Jobs that have not progressed to the ``STARTING`` state are
        cancelled.

        :param job_id: The Batch job ID of the job to terminate.
        :param reason: A message to attach to the job that explains the reason for canceling
        it.
        :returns: TerminateJobResponse
        :raises ClientException:
        :raises ServerException:
        """
        raise NotImplementedError

    @handler("UntagResource")
    def untag_resource(
        self, context: RequestContext, resource_arn: String, tag_keys: TagKeysList
    ) -> UntagResourceResponse:
        """Deletes specified tags from an Batch resource.

        :param resource_arn: The Amazon Resource Name (ARN) of the resource from which to delete
        tags.
        :param tag_keys: The keys of the tags to be removed.
        :returns: UntagResourceResponse
        :raises ClientException:
        :raises ServerException:
        """
        raise NotImplementedError

    @handler("UpdateComputeEnvironment")
    def update_compute_environment(
        self,
        context: RequestContext,
        compute_environment: String,
        state: CEState = None,
        unmanagedv_cpus: Integer = None,
        compute_resources: ComputeResourceUpdate = None,
        service_role: String = None,
        update_policy: UpdatePolicy = None,
    ) -> UpdateComputeEnvironmentResponse:
        """Updates an Batch compute environment.

        :param compute_environment: The name or full Amazon Resource Name (ARN) of the compute environment
        to update.
        :param state: The state of the compute environment.
        :param unmanagedv_cpus: The maximum number of vCPUs expected to be used for an unmanaged compute
        environment.
        :param compute_resources: Details of the compute resources managed by the compute environment.
        :param service_role: The full Amazon Resource Name (ARN) of the IAM role that allows Batch to
        make calls to other Amazon Web Services services on your behalf.
        :param update_policy: Specifies the updated infrastructure update policy for the compute
        environment.
        :returns: UpdateComputeEnvironmentResponse
        :raises ClientException:
        :raises ServerException:
        """
        raise NotImplementedError

    @handler("UpdateJobQueue")
    def update_job_queue(
        self,
        context: RequestContext,
        job_queue: String,
        state: JQState = None,
        scheduling_policy_arn: String = None,
        priority: Integer = None,
        compute_environment_order: ComputeEnvironmentOrders = None,
    ) -> UpdateJobQueueResponse:
        """Updates a job queue.

        :param job_queue: The name or the Amazon Resource Name (ARN) of the job queue.
        :param state: Describes the queue's ability to accept new jobs.
        :param scheduling_policy_arn: Amazon Resource Name (ARN) of the fair share scheduling policy.
        :param priority: The priority of the job queue.
        :param compute_environment_order: Details the set of compute environments mapped to a job queue and their
        order relative to each other.
        :returns: UpdateJobQueueResponse
        :raises ClientException:
        :raises ServerException:
        """
        raise NotImplementedError

    @handler("UpdateSchedulingPolicy")
    def update_scheduling_policy(
        self, context: RequestContext, arn: String, fairshare_policy: FairsharePolicy = None
    ) -> UpdateSchedulingPolicyResponse:
        """Updates a scheduling policy.

        :param arn: The Amazon Resource Name (ARN) of the scheduling policy to update.
        :param fairshare_policy: The fair share policy.
        :returns: UpdateSchedulingPolicyResponse
        :raises ClientException:
        :raises ServerException:
        """
        raise NotImplementedError



_A='RestApiId'
from localstack.services.cloudformation.deployment_utils import lambda_keys_to_lower
from localstack.services.cloudformation.service_models import GenericBaseModel
from localstack.utils.aws import aws_stack
class ApiGatewayAuthorizer(GenericBaseModel):
	@staticmethod
	def cloudformation_type():return'AWS::ApiGateway::Authorizer'
	def fetch_state(A,stack_name,resources):D=resources;C=stack_name;B=A.props;E=aws_stack.connect_to_service('apigateway');F=A.resolve_refs_recursively(C,B[_A],D);G=A.resolve_refs_recursively(C,B.get('AuthorizerUri'),D);H=E.get_authorizers(restApiId=F,limit=200)['items'];I=[A for A in H if A['type']==B.get('Type')and A.get('authorizerUri')==G];return(I or[None])[0]
	def get_physical_resource_id(A,attribute=None,**B):return A.props.get('id')
	@classmethod
	def get_deploy_templates(C):
		B='parameters';A='function'
		def D(params,resource_id,resources,**B):A=C(resources[resource_id]);return{'restApiId':A.props[_A],'authorizerId':A.props.get('id')}
		return{'create':{A:'create_authorizer',B:lambda_keys_to_lower()},'delete':{A:'delete_authorizer',B:D}}


_A='__dict__'
import logging,os
from typing import Any,Callable,List,OrderedDict,Set,Tuple
import dill
from localstack.utils.common import ObjectIdHashComparator
from localstack.utils.files import rm_rf
API_STATES_DIR='api_states'
KINESIS_DIR='kinesis'
DYNAMODB_DIR='dynamodb'
POD_KEEP='.podkeep'
LOG=logging.getLogger(__name__)
def check_already_visited(obj,visited):
	A=visited
	if hasattr(obj,_A):
		A=A or set();B=ObjectIdHashComparator(obj)
		if B in A:return True,A
		A.add(B)
	return False,A
def check_already_visited_obj_id(obj,visited):A=visited;A=A or set();B=id(obj);C=B in A;A.add(B);return C,A
def get_object_dict(obj):
	A=obj
	if isinstance(A,dict):return A
	B=getattr(A,_A,None);return B
def is_composite_type(obj):return isinstance(obj,(dict,OrderedDict))or hasattr(obj,_A)
def api_states_traverse(api_states_path,side_effect,mutables):
	C=side_effect
	for (A,K,F) in os.walk(api_states_path):
		for D in F:
			try:B=os.path.normpath(A).split(os.sep);G=B[-1];H=B[-2];I=B[-3];C(dir_name=A,fname=D,region=H,service_name=G,account_id=I,mutables=mutables)
			except Exception as J:
				E=f"Failed to apply {C.__name__} for {D} in dir {A}: {J}";LOG.warning(E)
				if LOG.isEnabledFor(logging.DEBUG):LOG.exception(E)
				continue
def load_persisted_object(state_file):
	A=state_file
	if not os.path.isfile(A):return
	import dill
	with open(A,'rb')as B:
		try:C=B.read();D=dill.loads(C);return D
		except Exception as E:LOG.debug('Unable to read pickled persistence file %s: %s',A,E)
def persist_object(obj,state_file):
	with open(state_file,'wb')as A:B=A.write(dill.dumps(obj));return B
def cleanse_keep_files(file_path):
	for (B,D,C) in os.walk(file_path):
		for A in C:
			if A==POD_KEEP:rm_rf(os.path.join(B,A))
def populate_empty_dirs(file_path):
	for (B,C,G) in os.walk(file_path):
		for D in C:
			A=os.path.join(B,D);E=list((B for B in os.listdir(A)if os.path.isfile(os.path.join(A,B))))
			if not E:
				F=os.path.join(A,POD_KEEP)
				with open(F,'w'):0


_G='appArn'
_F='parameters'
_E='function'
_D='create'
_C='BranchName'
_B='amplify'
_A='tags'
from localstack.services.cloudformation.deployment_utils import lambda_keys_to_lower,params_list_to_dict
from localstack.services.cloudformation.service_models import REF_ID_ATTRS,GenericBaseModel
from localstack.utils.aws import aws_stack
from localstack_ext.utils.aws import aws_utils
class AmplifyBranch(GenericBaseModel):
	@staticmethod
	def cloudformation_type():return'AWS::Amplify::Branch'
	def fetch_state(A,stack_name,resources):C=resources;B=stack_name;E=aws_stack.connect_to_service(_B);D=A.props;F=A.resolve_refs_recursively(B,D['AppId'],C);G=A.resolve_refs_recursively(B,D[_C],C);H=E.list_branches(appId=F)['branches'];I=[A for A in H if A['branchName']==G];return(I or[None])[0]
	def get_physical_resource_id(A,attribute,**B):return A.props.get(_C)
	@staticmethod
	def get_deploy_templates():
		def A(params,**C):
			A=params;A=lambda_keys_to_lower()(A,**C);B=A.pop('basicAuthConfig',{})
			if B:A['enableBasicAuth']=B['EnableBasicAuth'];A['basicAuthCredentials']='%s:%s'%(B['Username'],B['Password'])
			A[_A]=params_list_to_dict(_A,key_attr_name='key',value_attr_name='value')(A,**C);return A
		return{_D:{_E:'create_branch',_F:A}}
class AmplifyApp(GenericBaseModel):
	@staticmethod
	def cloudformation_type():return'AWS::Amplify::App'
	def get_cfn_attribute(B,attribute_name):
		A=attribute_name
		if A=='DefaultDomain':return'default.example.com'
		if A in REF_ID_ATTRS:return B.get_physical_resource_id(A)
		if A in['Arn']:return B.props.get(_G)
		return super(AmplifyApp,B).get_cfn_attribute(A)
	def get_physical_resource_id(C,attribute,**D):
		B=C.props;A=B.get('appId')
		if not A:return
		if attribute in REF_ID_ATTRS:return A
		return B.get(_G)or aws_utils.amplify_app_arn(A)
	@classmethod
	def fetch_details(C,app_name):B=aws_stack.connect_to_service(_B);A=B.list_apps()['apps'];A=[B for B in A if B['name']==app_name];return(A or[None])[0]
	def fetch_state(A,stack_name,resources):B=A.resolve_refs_recursively(stack_name,A.props['Name'],resources);return A.fetch_details(B)
	@staticmethod
	def get_deploy_templates():
		def A(params,**B):
			C='IAMServiceRole';A=params
			if C in A:A['iamServiceRoleArn']=A.pop(C)
			A=lambda_keys_to_lower()(A,**B);A[_A]=params_list_to_dict(_A,key_attr_name='key',value_attr_name='value')(A,**B);return A
		return{_D:{_E:'create_app',_F:A}}


_C='Name'
_B='cloudtrail'
_A='TrailName'
from localstack.services.cloudformation.service_models import REF_ID_ATTRS,GenericBaseModel
from localstack.utils.aws import aws_stack
class CloudTrail(GenericBaseModel):
	@staticmethod
	def cloudformation_type():return'AWS::CloudTrail::Trail'
	def fetch_state(B,stack_name,resources):C=aws_stack.connect_to_service(_B);A=[A for A in C.list_trails()['Trails']if A[_C]==B.props[_A]];return(A or[None])[0]
	def get_physical_resource_id(A,attribute,**B):
		if attribute in REF_ID_ATTRS:return A.props[_A]
	@staticmethod
	def get_deploy_templates():
		K='SnsTopicName';J='S3KeyPrefix';I='S3BucketName';H='IsMultiRegionTrail';G='IncludeGlobalServiceEvents';F='EnableLogFileValidation';E='CloudWatchLogsRoleArn';D='CloudWatchLogsLogGroupArn';C='parameters';B='Properties';A='function'
		def L(resource_id,resources,*F,**G):
			D=resources[resource_id];A=D.get(B,{});C=A.get('EventSelectors',[])
			if C:E=aws_stack.connect_to_service(_B);E.put_event_selectors(TrailName=A[_A],EventSelectors=C)
			return{}
		def M(resource_id,resources,*E,**F):
			C=resources[resource_id];A=C.get(B,{})
			if A.get('IsLogging'):D=aws_stack.connect_to_service(_B);D.start_logging(Name=A[_A])
			return{}
		return{'create':[{A:'create_trail',C:{D:D,E:E,F:F,G:G,H:H,'KmsKeyId':'KMSKeyId',_C:_A,I:I,J:J,K:K,'TagsList':'Tags'}},{A:L},{A:M}],'delete':{A:'delete_trail',C:{_C:_A}}}


_M='LoadBalancerName'
_L='LoadBalancers'
_K='ListenerArn'
_J='Protocol'
_I='TargetGroupArn'
_H='parameters'
_G='create'
_F=None
_E='LoadBalancerArn'
_D='Name'
_C='function'
_B='Port'
_A='elbv2'
from localstack import config
from localstack.services.cloudformation.service_models import GenericBaseModel
from localstack.utils.aws import aws_stack
from localstack.utils.common import clone,items_equivalent
from localstack_ext import config as ext_config
from localstack_ext.services.cloudformation.service_models import LOG
class ELBV2TargetGroup(GenericBaseModel):
	@staticmethod
	def cloudformation_type():return'AWS::ElasticLoadBalancingV2::TargetGroup'
	def get_physical_resource_id(A,attribute=_F,**B):return A.props.get(_I)
	def fetch_state(B,stack_name,resources):C=aws_stack.connect_to_service(_A);D=B.resolve_refs_recursively(stack_name,B.props.get(_D),resources);A=C.describe_target_groups()['TargetGroups'];A=[B for B in A if B.get('TargetGroupName')==D];return(A or[_F])[0]
	@classmethod
	def get_deploy_templates(B):
		def A(resource_id,resources,resource_type,func,stack_name,*H,**I):
			A=resources;C=B(A[resource_id]);C.fetch_and_update_state(stack_name,A);D=C.props;E=D.get('Targets')
			if E:F=aws_stack.connect_to_service(_A);G=D.get(_I);F.register_targets(TargetGroupArn=G,Targets=E)
		def C(resource_id,resources,*I,**J):
			E='Value';F=B(resources[resource_id]);C=F.props;A=C.get('TargetGroupAttributes')
			if A:
				G=aws_stack.connect_to_service(_A);H=C.get(_I)
				for D in A:D[E]=str(D.get(E,''))
				G.modify_target_group_attributes(TargetGroupArn=H,Attributes=A)
		D=[_D,_J,'ProtocolVersion',_B,'VpcId','HealthCheckProtocol','HealthCheckPort','HealthCheckEnabled','HealthCheckPath','HealthCheckIntervalSeconds','HealthCheckTimeoutSeconds','HealthyThresholdCount','UnhealthyThresholdCount','Matcher','TargetType','Tags'];return{_G:[{_C:'create_target_group',_H:D},{_C:A},{_C:C}]}
class ELBV2ListenerRule(GenericBaseModel):
	@staticmethod
	def cloudformation_type():return'AWS::ElasticLoadBalancingV2::ListenerRule'
	def get_physical_resource_id(A,attribute,**B):return A.props.get('RuleArn')
	def fetch_state(D,stack_name,resources):
		F='Conditions';E='Actions';B=False
		def G(rule):
			def A(action,_action):
				E='AuthenticateCognitoConfig';D=_action;C=action;F=C.get(E,{})
				for A in ['Type','Order']:
					if C.get(A)!=D.get(A):return B
				G=D.get(E,{})
				for A in ['UserPoolArn','UserPoolClientId','UserPoolDomain']:
					if F.get(A)!=G.get(A):return B
				return True
			C=rule.get(E,[]);return items_equivalent(K,C,A)
		def H(rule):
			def A(cond,_cond):
				F='Values';E='Field';C=cond;A=_cond;G=([C.get(A)for A in D if C.get(A)]+[C])[0]
				if C.get(E)!=A.get(E):return B
				for H in D:A=A.get(H)or A
				I=G.get(F,[])
				if I!=A.get(F):return B
				return True
			C=rule.get(F,[]);D=['HostHeaderConfig','HttpHeaderConfig','HttpRequestMethodConfig','PathPatternConfig','QueryStringConfig','SourceIpConfig'];return items_equivalent(J,C,A)
		I=aws_stack.connect_to_service(_A);A=D.props;J=A.get(F,[]);K=A.get(E,[]);L=D.resolve_refs_recursively(stack_name,A.get(_K),resources);C=I.describe_rules(ListenerArn=L)['Rules'];C=[A for A in C if G(A)and H(A)];return(C or[_F])[0]
	@staticmethod
	def get_deploy_templates():return{_G:{_C:'create_rule'}}
class ELBV2Listener(GenericBaseModel):
	@staticmethod
	def cloudformation_type():return'AWS::ElasticLoadBalancingV2::Listener'
	def get_physical_resource_id(A,attribute,**B):return A.props.get(_K)
	def fetch_state(A,stack_name,resources):
		D=resources;C=stack_name;H=aws_stack.connect_to_service(_A);B=A.props;F=A.resolve_refs_recursively(C,B.get(_E),D);I=A.resolve_refs_recursively(C,B.get(_J),D);J=A.resolve_refs_recursively(C,B.get(_B),D);E=H.describe_listeners(LoadBalancerArn=F)['Listeners'];K=[str(config.EDGE_PORT),str(config.EDGE_PORT_HTTP)];G=[A for A in E if A.get(_E)==F and(str(J or'')in[str(A.get(_B)),'']or str(A.get(_B))in K)and A.get(_J)==I]
		if E and not G:LOG.debug('No matching entry when filtering ELBv2 listeners %s for props %s'%(E,B))
		return(G or[_F])[0]
	@staticmethod
	def get_deploy_templates():
		def A(params,**E):
			C='StatusCode';B=clone(params)
			for D in B.get('DefaultActions',[]):A=D.get('RedirectConfig',{});A[C]=A.get(C)or'HTTP_301';A[_B]=A.get(_B)and str(A[_B])
			return B
		return{_G:{_C:'create_listener',_H:A}}
class ELBV2LoadBalancer(GenericBaseModel):
	@staticmethod
	def cloudformation_type():return'AWS::ElasticLoadBalancingV2::LoadBalancer'
	def get_physical_resource_id(A,attribute,**B):return A.props.get(_E)
	def fetch_state(C,stack_name,resources):A=C.props.get(_D);A=C.resolve_refs_recursively(stack_name,A,resources);D=aws_stack.connect_to_service(_A);B=D.describe_load_balancers()[_L];B=[C for C in B if C[_M]==A];return(B or[_F])[0]
	def get_cfn_attribute(A,attribute_name):
		B=attribute_name
		if B=='DNSName':C=A.props.get(_E);return'%s.elb.%s'%(C.split('/')[2],ext_config.RESOURCES_BASE_DOMAIN_NAME)
		return super(ELBV2LoadBalancer,A).get_cfn_attribute(B)
	@staticmethod
	def get_deploy_templates():
		C='resource_id';B='LoadBalancerAttributes'
		def A(params,**D):A=clone(params);A.pop(B,[]);A[_D]=A.get(_D)or D.get(C);return A
		def D(params,**F):D=params;E=D.get(_D)or F.get(C);A=aws_stack.connect_to_service(_A).describe_load_balancers()[_L];A=[B for B in A if B[_M]==E];A={_E:A and A[0][_E]or E,'Attributes':D.get(B,[])};return A
		return{_G:[{_C:'create_load_balancer',_H:A},{_C:'modify_load_balancer_attributes',_H:D}]}